@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  slurm_persist_conn.h - Definitions for communicating over a persistent
+ *  slurm_persist_conn.c - Definitions for communicating over a persistent
  *                         connection within Slurm.
  ******************************************************************************
  *  Copyright (C) 2016 SchedMD LLC
@@ -47,7 +47,7 @@
 #include "slurm/slurm_errno.h"
 #include "src/common/fd.h"
 #include "src/common/macros.h"
-#include "src/common/slurm_auth.h"
+#include "src/interfaces/auth.h"
 #include "src/common/slurm_protocol_pack.h"
 #include "src/common/slurmdbd_defs.h"
 #include "src/common/slurmdbd_pack.h"
@@ -56,12 +56,6 @@
 
 #define MAX_THREAD_COUNT 100
 
-/*
- *  Maximum message size. Messages larger than this value (in bytes)
- *  will not be received.
- */
-#define MAX_MSG_SIZE     (16*1024*1024)
-
 typedef struct {
 	void *arg;
 	slurm_persist_conn_t *conn;
@@ -75,8 +69,8 @@ static pthread_mutex_t thread_count_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t  thread_count_cond = PTHREAD_COND_INITIALIZER;
 static time_t          shutdown_time = 0;
 
-static Buf _slurm_persist_recv_msg(slurm_persist_conn_t *persist_conn,
-                                   bool reopen);
+static buf_t *_slurm_persist_recv_msg(slurm_persist_conn_t *persist_conn,
+				      bool reopen);
 
 /* Return time in msec since "start time" */
 static int _tot_wait (struct timeval *start_time)
@@ -90,7 +84,7 @@ static int _tot_wait (struct timeval *start_time)
 	return msec_delay;
 }
 
-/* close and fd and replace it with a -1 */
+/* close an fd and replace it with a -1 */
 static void _close_fd(int *fd)
 {
 	if (*fd && *fd >= 0) {
@@ -227,7 +221,7 @@ static int _process_service_connection(
 	char *msg_char = NULL;
 	ssize_t msg_read = 0, offset = 0;
 	bool first = true, fini = false;
-	Buf buffer = NULL;
+	buf_t *buffer = NULL;
 	int rc = SLURM_SUCCESS;
 
 	xassert(persist_conn->callback_proc);
@@ -246,15 +240,13 @@ static int _process_service_connection(
 		if (msg_read == 0)	/* EOF */
 			break;
 		if (msg_read != sizeof(nw_size)) {
-			error("Could not read msg_size from "
-			      "connection %d(%s) uid(%d)",
+			error("Could not read msg_size from connection %d(%s) uid(%u)",
 			      persist_conn->fd, persist_conn->rem_host, uid);
 			break;
 		}
 		msg_size = ntohl(nw_size);
 		if ((msg_size < 2) || (msg_size > MAX_MSG_SIZE)) {
-			error("Invalid msg_size (%u) from "
-			      "connection %d(%s) uid(%d)",
+			error("Invalid msg_size (%u) from connection %d(%s) uid(%u)",
 			      msg_size, persist_conn->fd,
 			      persist_conn->rem_host, uid);
 			break;
@@ -289,8 +281,7 @@ static int _process_service_connection(
 				    rc != ACCOUNTING_FIRST_REG &&
 				    rc != ACCOUNTING_TRES_CHANGE_DB &&
 				    rc != ACCOUNTING_NODES_CHANGE_DB) {
-					error("Processing last message from "
-					      "connection %d(%s) uid(%d)",
+					error("Processing last message from connection %d(%s) uid(%u)",
 					      persist_conn->fd,
 					      persist_conn->rem_host, uid);
 					if (rc == ESLURM_ACCESS_DENIED ||
@@ -314,17 +305,17 @@ static int _process_service_connection(
 				 * deal as the slurmctld will just send the
 				 * message again. */
 				if (persist_conn->rem_port)
-					log_flag(NET, "%s: Problem sending response to connection host:%s fd:%d uid:%d",
+					log_flag(NET, "%s: Problem sending response to connection host:%s fd:%d uid:%u",
 						 __func__,
 						 persist_conn->rem_host,
 						 persist_conn->fd, uid);
 				fini = true;
 			}
-			free_buf(buffer);
+			FREE_NULL_BUFFER(buffer);
 		}
 	}
 
-	log_flag(NET, "%s: Closed connection host:%s fd:%d uid:%d",
+	log_flag(NET, "%s: Closed connection host:%s fd:%d uid:%u",
 		 __func__, persist_conn->rem_host, persist_conn->fd, uid);
 
 	return rc;
@@ -578,7 +569,6 @@ extern int slurm_persist_conn_open_without_init(
 		return SLURM_ERROR;
 	}
 	fd_set_nonblocking(persist_conn->fd);
-	fd_set_close_on_exec(persist_conn->fd);
 
 	return SLURM_SUCCESS;
 }
@@ -627,7 +617,7 @@ extern int slurm_persist_conn_open(slurm_persist_conn_t *persist_conn)
 		      __func__, persist_conn->rem_host, persist_conn->rem_port);
 		_close_fd(&persist_conn->fd);
 	} else {
-		Buf buffer = _slurm_persist_recv_msg(persist_conn, false);
+		buf_t *buffer = _slurm_persist_recv_msg(persist_conn, false);
 		persist_msg_t msg;
 		slurm_persist_conn_t persist_conn_tmp;
 
@@ -646,7 +636,7 @@ extern int slurm_persist_conn_open(slurm_persist_conn_t *persist_conn)
 		 * communication . */
 		persist_conn_tmp.flags &= (~PERSIST_FLAG_DBD);
 		rc = slurm_persist_msg_unpack(&persist_conn_tmp, &msg, buffer);
-		free_buf(buffer);
+		FREE_NULL_BUFFER(buffer);
 
 		resp = (persist_rc_msg_t *)msg.data;
 		if (resp && (rc == SLURM_SUCCESS)) {
@@ -707,7 +697,7 @@ extern void slurm_persist_conn_members_destroy(
 	slurm_persist_conn_close(persist_conn);
 
 	if (persist_conn->auth_cred) {
-		g_slurm_auth_destroy(persist_conn->auth_cred);
+		auth_g_destroy(persist_conn->auth_cred);
 		persist_conn->auth_cred = NULL;
 	}
 	xfree(persist_conn->cluster_name);
@@ -726,10 +716,10 @@ extern void slurm_persist_conn_destroy(slurm_persist_conn_t *persist_conn)
 extern int slurm_persist_conn_process_msg(slurm_persist_conn_t *persist_conn,
 					  persist_msg_t *persist_msg,
 					  char *msg_char, uint32_t msg_size,
-					  Buf *out_buffer, bool first)
+					  buf_t **out_buffer, bool first)
 {
 	int rc;
-	Buf recv_buffer = NULL;
+	buf_t *recv_buffer = NULL;
 	char *comment = NULL;
 
 	/* puts msg_char into buffer struct */
@@ -837,12 +827,17 @@ extern int slurm_persist_conn_writeable(slurm_persist_conn_t *persist_conn)
 		}
 		if (ufds.revents & POLLERR) {
 			if (_comm_fail_log(persist_conn)) {
-				if (fd_get_socket_error(persist_conn->fd, &errno))
-					error("%s: unable to get error for persistent connection %d: %m",
-					      __func__, persist_conn->fd);
+				int rc, err;
+				if ((rc = fd_get_socket_error(persist_conn->fd,
+							      &err)))
+					error("%s: unable to get error for persistent connection %d: %s",
+					      __func__, persist_conn->fd,
+					      strerror(rc));
 				else
-					error("%s: persistent connection %d experienced an error: %m",
-					      __func__, persist_conn->fd);
+					error("%s: persistent connection %d experienced an error: %s",
+					      __func__, persist_conn->fd,
+					      strerror(err));
+				slurm_seterrno(err);
 			}
 			if (persist_conn->trigger_callbacks.dbd_fail)
 				(persist_conn->trigger_callbacks.dbd_fail)();
@@ -860,8 +855,8 @@ extern int slurm_persist_conn_writeable(slurm_persist_conn_t *persist_conn)
 	return 0;
 }
 
-extern int slurm_persist_send_msg(
-	slurm_persist_conn_t *persist_conn, Buf buffer)
+extern int slurm_persist_send_msg(slurm_persist_conn_t *persist_conn,
+				  buf_t *buffer)
 {
 	uint32_t msg_size, nw_size;
 	char *msg;
@@ -919,19 +914,19 @@ extern int slurm_persist_send_msg(
 	return SLURM_SUCCESS;
 }
 
-static Buf _slurm_persist_recv_msg(slurm_persist_conn_t *persist_conn,
-				   bool reopen)
+static buf_t *_slurm_persist_recv_msg(slurm_persist_conn_t *persist_conn,
+				      bool reopen)
 {
 	uint32_t msg_size, nw_size;
 	char *msg;
 	ssize_t msg_read, offset;
-	Buf buffer;
+	buf_t *buffer;
 
 	xassert(persist_conn);
 
 	if (persist_conn->fd < 0) {
 		if (!persist_conn->shutdown || *persist_conn->shutdown)
-			log_flag(NET, "%s: Invalid file descriptor fd:%d host:%s port:%"PRIu16,
+			log_flag(NET, "%s: Invalid file descriptor fd:%d host:%s port:%u",
 				 __func__, persist_conn->fd,
 				 persist_conn->rem_host,
 				 persist_conn->rem_port);
@@ -1005,15 +1000,15 @@ endit:
 	return NULL;
 }
 
-extern Buf slurm_persist_recv_msg(slurm_persist_conn_t *persist_conn)
+extern buf_t *slurm_persist_recv_msg(slurm_persist_conn_t *persist_conn)
 {
 	return _slurm_persist_recv_msg(persist_conn, true);
 }
 
-extern Buf slurm_persist_msg_pack(slurm_persist_conn_t *persist_conn,
-				  persist_msg_t *req_msg)
+extern buf_t *slurm_persist_msg_pack(slurm_persist_conn_t *persist_conn,
+				     persist_msg_t *req_msg)
 {
-	Buf buffer;
+	buf_t *buffer;
 
 	xassert(persist_conn);
 
@@ -1033,7 +1028,7 @@ extern Buf slurm_persist_msg_pack(slurm_persist_conn_t *persist_conn,
 
 		pack16(req_msg->msg_type, buffer);
 		if (pack_msg(&msg, buffer) != SLURM_SUCCESS) {
-			free_buf(buffer);
+			FREE_NULL_BUFFER(buffer);
 			return NULL;
                 }
 	}
@@ -1043,7 +1038,7 @@ extern Buf slurm_persist_msg_pack(slurm_persist_conn_t *persist_conn,
 
 
 extern int slurm_persist_msg_unpack(slurm_persist_conn_t *persist_conn,
-				    persist_msg_t *resp_msg, Buf buffer)
+				    persist_msg_t *resp_msg, buf_t *buffer)
 {
 	int rc;
 
@@ -1076,7 +1071,7 @@ extern int slurm_persist_msg_unpack(slurm_persist_conn_t *persist_conn,
 	if (resp_msg->msg_type == REQUEST_PERSIST_INIT) {
 		slurm_msg_t *msg = resp_msg->data;
 		if (persist_conn->auth_cred)
-			g_slurm_auth_destroy(persist_conn->auth_cred);
+			auth_g_destroy(persist_conn->auth_cred);
 
 		persist_conn->auth_cred = msg->auth_cred;
 		msg->auth_cred = NULL;
@@ -1087,8 +1082,8 @@ unpack_error:
 	return SLURM_ERROR;
 }
 
-extern void slurm_persist_pack_init_req_msg(
-	persist_init_req_msg_t *msg, Buf buffer)
+extern void slurm_persist_pack_init_req_msg(persist_init_req_msg_t *msg,
+					    buf_t *buffer)
 {
 	/* always send version field first for backwards compatibility */
 	pack16(msg->version, buffer);
@@ -1103,8 +1098,8 @@ extern void slurm_persist_pack_init_req_msg(
 	}
 }
 
-extern int slurm_persist_unpack_init_req_msg(
-	persist_init_req_msg_t **msg, Buf buffer)
+extern int slurm_persist_unpack_init_req_msg(persist_init_req_msg_t **msg,
+					     buf_t *buffer)
 {
 	uint32_t tmp32;
 
@@ -1141,8 +1136,9 @@ extern void slurm_persist_free_init_req_msg(persist_init_req_msg_t *msg)
 	}
 }
 
-extern void slurm_persist_pack_rc_msg(
-	persist_rc_msg_t *msg, Buf buffer, uint16_t protocol_version)
+extern void slurm_persist_pack_rc_msg(persist_rc_msg_t *msg,
+				      buf_t *buffer,
+				      uint16_t protocol_version)
 {
 	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		packstr(msg->comment, buffer);
@@ -1155,8 +1151,9 @@ extern void slurm_persist_pack_rc_msg(
 	}
 }
 
-extern int slurm_persist_unpack_rc_msg(
-	persist_rc_msg_t **msg, Buf buffer, uint16_t protocol_version)
+extern int slurm_persist_unpack_rc_msg(persist_rc_msg_t **msg,
+				       buf_t *buffer,
+				       uint16_t protocol_version)
 {
 	uint32_t uint32_tmp;
 
@@ -1191,9 +1188,9 @@ extern void slurm_persist_free_rc_msg(persist_rc_msg_t *msg)
 	}
 }
 
-extern Buf slurm_persist_make_rc_msg(slurm_persist_conn_t *persist_conn,
-				     uint32_t rc, char *comment,
-				     uint16_t ret_info)
+extern buf_t *slurm_persist_make_rc_msg(slurm_persist_conn_t *persist_conn,
+					uint32_t rc, char *comment,
+					uint16_t ret_info)
 {
 	persist_rc_msg_t msg;
 	persist_msg_t resp;
@@ -1211,10 +1208,10 @@ extern Buf slurm_persist_make_rc_msg(slurm_persist_conn_t *persist_conn,
 	return slurm_persist_msg_pack(persist_conn, &resp);
 }
 
-extern Buf slurm_persist_make_rc_msg_flags(slurm_persist_conn_t *persist_conn,
-					   uint32_t rc, char *comment,
-					   uint16_t flags,
-					   uint16_t ret_info)
+extern buf_t *slurm_persist_make_rc_msg_flags(slurm_persist_conn_t *persist_conn,
+					      uint32_t rc, char *comment,
+					      uint16_t flags,
+					      uint16_t ret_info)
 {
 	persist_rc_msg_t msg;
 	persist_msg_t resp;
