@@ -41,7 +41,6 @@
 
 #include "src/common/fd.h"
 #include "src/common/slurmdbd_pack.h"
-#include "src/common/xsignal.h"
 #include "src/common/xstring.h"
 
 #include "slurmdbd_agent.h"
@@ -55,7 +54,6 @@ slurm_persist_conn_t *slurmdbd_conn = NULL;
 
 
 #define DBD_MAGIC		0xDEAD3219
-#define SLURMDBD_TIMEOUT	900	/* Seconds SlurmDBD for response */
 #define DEBUG_PRINT_MAX_MSG_TYPES 10
 #define MAX_DBD_DEFAULT_ACTION MAX_DBD_ACTION_DISCARD
 
@@ -73,7 +71,7 @@ static pthread_cond_t  slurmdbd_cond = PTHREAD_COND_INITIALIZER;
 
 static int max_dbd_msg_action = MAX_DBD_DEFAULT_ACTION;
 
-static int _unpack_return_code(uint16_t rpc_version, Buf buffer)
+static int _unpack_return_code(uint16_t rpc_version, buf_t *buffer)
 {
 	uint16_t msg_type = -1;
 	persist_rc_msg_t *msg;
@@ -144,24 +142,24 @@ static int _unpack_return_code(uint16_t rpc_version, Buf buffer)
 static int _get_return_code(void)
 {
 	int rc = SLURM_ERROR;
-	Buf buffer = slurm_persist_recv_msg(slurmdbd_conn);
+	buf_t *buffer = slurm_persist_recv_msg(slurmdbd_conn);
 	if (buffer == NULL)
 		return rc;
 
 	rc = _unpack_return_code(slurmdbd_conn->version, buffer);
 
-	free_buf(buffer);
+	FREE_NULL_BUFFER(buffer);
 	return rc;
 }
 
 static int _handle_mult_rc_ret(void)
 {
-	Buf buffer;
+	buf_t *buffer;
 	uint16_t msg_type;
 	persist_rc_msg_t *msg = NULL;
 	dbd_list_msg_t *list_msg = NULL;
 	int rc = SLURM_ERROR;
-	Buf out_buf = NULL;
+	buf_t *out_buf = NULL;
 
 	buffer = slurm_persist_recv_msg(slurmdbd_conn);
 	if (buffer == NULL)
@@ -183,14 +181,14 @@ static int _handle_mult_rc_ret(void)
 			ListIterator itr =
 				list_iterator_create(list_msg->my_list);
 			while ((out_buf = list_next(itr))) {
-				Buf b;
+				buf_t *b;
 				if ((rc = _unpack_return_code(
 					     slurmdbd_conn->version, out_buf))
 				    != SLURM_SUCCESS)
 					break;
 
 				if ((b = list_dequeue(agent_list))) {
-					free_buf(b);
+					FREE_NULL_BUFFER(b);
 				} else {
 					error("DBD_GOT_MULT_MSG "
 					      "unpack message error");
@@ -238,31 +236,31 @@ static int _handle_mult_rc_ret(void)
 	}
 
 unpack_error:
-	free_buf(buffer);
+	FREE_NULL_BUFFER(buffer);
 	return rc;
 }
 
 /****************************************************************************
  * Functions for agent to manage queue of pending message for the Slurm DBD
  ****************************************************************************/
-static Buf _load_dbd_rec(int fd)
+static buf_t *_load_dbd_rec(int fd)
 {
 	ssize_t size, rd_size;
 	uint32_t msg_size, magic;
 	char *msg;
-	Buf buffer;
+	buf_t *buffer;
 
 	size = sizeof(msg_size);
 	rd_size = read(fd, &msg_size, size);
 	if (rd_size == 0)
-		return (Buf) NULL;
+		return NULL;
 	if (rd_size != size) {
 		error("state recover error: %m");
-		return (Buf) NULL;
+		return NULL;
 	}
-	if (msg_size > MAX_DBD_MSG_LEN) {
+	if (msg_size > MAX_BUF_SIZE) {
 		error("state recover error, msg_size=%u", msg_size);
-		return (Buf) NULL;
+		return NULL;
 	}
 
 	buffer = init_buf((int) msg_size);
@@ -278,8 +276,8 @@ static Buf _load_dbd_rec(int fd)
 			continue;
 		else {
 			error("state recover error: %m");
-			free_buf(buffer);
-			return (Buf) NULL;
+			FREE_NULL_BUFFER(buffer);
+			return NULL;
 		}
 	}
 
@@ -287,8 +285,8 @@ static Buf _load_dbd_rec(int fd)
 	rd_size = read(fd, &magic, size);
 	if ((rd_size != size) || (magic != DBD_MAGIC)) {
 		error("state recover error");
-		free_buf(buffer);
-		return (Buf) NULL;
+		FREE_NULL_BUFFER(buffer);
+		return NULL;
 	}
 
 	return buffer;
@@ -297,7 +295,7 @@ static Buf _load_dbd_rec(int fd)
 static void _load_dbd_state(void)
 {
 	char *dbd_fname = NULL;
-	Buf buffer;
+	buf_t *buffer;
 	int fd, recovered = 0;
 	uint16_t rpc_version = 0;
 
@@ -324,7 +322,7 @@ static void _load_dbd_state(void)
 		safe_unpackstr_xmalloc(&ver_str, &ver_str_len, buffer);
 		debug3("Version string in dbd_state header is %s", ver_str);
 	unpack_error:
-		free_buf(buffer);
+		FREE_NULL_BUFFER(buffer);
 		buffer = NULL;
 		if (ver_str) {
 			/* get the version after VER */
@@ -351,7 +349,7 @@ static void _load_dbd_state(void)
 				set_buf_offset(buffer, 0);
 				rc = unpack_slurmdbd_msg(
 					&msg, rpc_version, buffer);
-				free_buf(buffer);
+				FREE_NULL_BUFFER(buffer);
 				if (rc == SLURM_SUCCESS)
 					buffer = pack_slurmdbd_msg(
 						&msg, SLURM_PROTOCOL_VERSION);
@@ -362,8 +360,7 @@ static void _load_dbd_state(void)
 				error("no buffer given");
 				continue;
 			}
-			if (!list_enqueue(agent_list, buffer))
-				fatal("list_enqueue, no memory");
+			list_enqueue(agent_list, buffer);
 			recovered++;
 			buffer = NULL;
 		}
@@ -375,7 +372,7 @@ static void _load_dbd_state(void)
 	xfree(dbd_fname);
 }
 
-static int _save_dbd_rec(int fd, Buf buffer)
+static int _save_dbd_rec(int fd, buf_t *buffer)
 {
 	ssize_t size, wrote;
 	uint32_t msg_size = get_buf_offset(buffer);
@@ -416,7 +413,7 @@ static int _save_dbd_rec(int fd, Buf buffer)
 static void _save_dbd_state(void)
 {
 	char *dbd_fname = NULL;
-	Buf buffer;
+	buf_t *buffer;
 	int fd, rc, wrote = 0;
 	uint16_t msg_type;
 	uint32_t offset;
@@ -433,7 +430,7 @@ static void _save_dbd_state(void)
 		buffer = init_buf(strlen(curr_ver_str));
 		packstr(curr_ver_str, buffer);
 		rc = _save_dbd_rec(fd, buffer);
-		free_buf(buffer);
+		FREE_NULL_BUFFER(buffer);
 		if (rc != SLURM_SUCCESS)
 			goto end_it;
 
@@ -446,19 +443,19 @@ static void _save_dbd_state(void)
 			 */
 			offset = get_buf_offset(buffer);
 			if (offset < 2) {
-				free_buf(buffer);
+				FREE_NULL_BUFFER(buffer);
 				continue;
 			}
 			set_buf_offset(buffer, 0);
 			(void) unpack16(&msg_type, buffer);  /* checked by offset */
 			set_buf_offset(buffer, offset);
 			if (msg_type == DBD_REGISTER_CTLD) {
-				free_buf(buffer);
+				FREE_NULL_BUFFER(buffer);
 				continue;
 			}
 
 			rc = _save_dbd_rec(fd, buffer);
-			free_buf(buffer);
+			FREE_NULL_BUFFER(buffer);
 			if (rc != SLURM_SUCCESS)
 				break;
 			wrote++;
@@ -475,65 +472,43 @@ end_it:
 	xfree(dbd_fname);
 }
 
-/* Purge queued step records from the agent queue
- * RET number of records purged */
-static int _purge_step_req(void)
+/*
+ * Purge queued records from the agent queue
+ */
+static int _purge_agent_list_req(void *x, void *arg)
 {
-	int purged = 0;
-	ListIterator iter;
 	uint16_t msg_type;
 	uint32_t offset;
-	Buf buffer;
+	buf_t *buffer = x;
+	uint16_t purge_type = *(uint16_t *)arg;
 
-	iter = list_iterator_create(agent_list);
-	while ((buffer = list_next(iter))) {
-		offset = get_buf_offset(buffer);
-		if (offset < 2)
-			continue;
-		set_buf_offset(buffer, 0);
-		(void) unpack16(&msg_type, buffer);	/* checked by offset */
-		set_buf_offset(buffer, offset);
+	offset = get_buf_offset(buffer);
+	if (offset < 2)
+		return 0;
+	set_buf_offset(buffer, 0);
+	(void) unpack16(&msg_type, buffer);	/* checked by offset */
+	set_buf_offset(buffer, offset);
+	switch (purge_type) {
+	case DBD_STEP_START:
 		if ((msg_type == DBD_STEP_START) ||
-		    (msg_type == DBD_STEP_COMPLETE)) {
-			list_remove(iter);
-			purged++;
-		}
+		    (msg_type == DBD_STEP_COMPLETE))
+			return 1;
+		break;
+	case DBD_JOB_START:
+		if (msg_type == DBD_JOB_START)
+			return 1;
+		break;
+	default:
+		error("unknown purge type %d", purge_type);
+		break;
 	}
-	list_iterator_destroy(iter);
-	info("purge %d step records", purged);
-	return purged;
-}
 
-/* Purge queued job start records from the agent queue
- * RET number of records purged */
-static int _purge_job_start_req(void)
-{
-	int purged = 0;
-	ListIterator iter;
-	uint16_t msg_type;
-	uint32_t offset;
-	Buf buffer;
-
-	iter = list_iterator_create(agent_list);
-	while ((buffer = list_next(iter))) {
-		offset = get_buf_offset(buffer);
-		if (offset < 2)
-			continue;
-		set_buf_offset(buffer, 0);
-		(void) unpack16(&msg_type, buffer);	/* checked by offset */
-		set_buf_offset(buffer, offset);
-		if (msg_type == DBD_JOB_START) {
-			list_remove(iter);
-			purged++;
-		}
-	}
-	list_iterator_destroy(iter);
-	info("purge %d job start records", purged);
-	return purged;
+	return 0;
 }
 
 static void _max_dbd_msg_action(uint32_t *msg_cnt)
 {
+	int purged = 0;
 	if (max_dbd_msg_action == MAX_DBD_ACTION_EXIT) {
 		if (*msg_cnt < slurm_conf.max_dbd_msgs)
 			return;
@@ -544,19 +519,25 @@ static void _max_dbd_msg_action(uint32_t *msg_cnt)
 	}
 
 	/* MAX_DBD_ACTION_DISCARD */
-	if (*msg_cnt >= (slurm_conf.max_dbd_msgs - 1))
-		*msg_cnt -= _purge_step_req();
-	if (*msg_cnt >= (slurm_conf.max_dbd_msgs - 1))
-		*msg_cnt -= _purge_job_start_req();
-}
-
-static void _sig_handler(int signal)
-{
+	if (*msg_cnt >= (slurm_conf.max_dbd_msgs - 1)) {
+		uint16_t purge_type = DBD_STEP_START;
+		purged = list_delete_all(agent_list, _purge_agent_list_req,
+					 &purge_type);
+		*msg_cnt -= purged;
+		info("purge %d step records", purged);
+	}
+	if (*msg_cnt >= (slurm_conf.max_dbd_msgs - 1)) {
+		uint16_t purge_type = DBD_JOB_START;
+		purged = list_delete_all(agent_list, _purge_agent_list_req,
+					 &purge_type);
+		*msg_cnt -= purged;
+		info("purge %d job start records", purged);
+	}
 }
 
 static int _print_agent_list_msg_type(void *x, void *arg)
 {
-	Buf buffer = (Buf) x;
+	buf_t *buffer = (buf_t *) x;
 	char *mlist = (char *) arg;
 	uint16_t msg_type;
 	uint32_t offset = get_buf_offset(buffer);
@@ -584,7 +565,7 @@ static void _print_agent_list_msg_types(void)
 
 	if ((processed = list_for_each_max(agent_list, &max_msgs,
 					   _print_agent_list_msg_type,
-					   mlist, true)) < 0) {
+					   mlist, true, true)) < 0) {
 		error("unable to create msg type list");
 		xfree(mlist);
 		return;
@@ -604,10 +585,9 @@ static void *_agent(void *x)
 {
 	int rc;
 	uint32_t cnt;
-	Buf buffer;
+	buf_t *buffer;
 	struct timespec abs_time;
 	static time_t fail_time = 0;
-	int sigarray[] = {SIGUSR1, 0};
 	persist_msg_t list_req = {0};
 	dbd_list_msg_t list_msg;
 	DEF_TIMERS;
@@ -621,11 +601,6 @@ static void *_agent(void *x)
 	list_req.data = &list_msg;
 	memset(&list_msg, 0, sizeof(dbd_list_msg_t));
 
-	/* Prepare to catch SIGUSR1 to interrupt pending
-	 * I/O and terminate in a timely fashion. */
-	xsignal(SIGUSR1, _sig_handler);
-	xsignal_unblock(sigarray);
-
 	log_flag(AGENT, "slurmdbd agent_count=%d with msg_type=%s",
 		 list_count(agent_list),
 		 slurmdbd_msg_type_2_str(list_req.msg_type, 1));
@@ -672,27 +647,26 @@ static void *_agent(void *x)
 			info("agent_count:%d", cnt);
 		/* Leave item on the queue until processing complete */
 		if (agent_list) {
-			int handle_agent_count = 1000;
-			if (cnt > handle_agent_count) {
+			uint32_t msg_size = sizeof(list_req);
+			if (cnt > 1) {
 				int agent_count = 0;
 				ListIterator agent_itr =
 					list_iterator_create(agent_list);
 				list_msg.my_list = list_create(NULL);
 				while ((buffer = list_next(agent_itr))) {
+					msg_size += size_buf(buffer);
+					if (msg_size > MAX_MSG_SIZE)
+						break;
 					list_enqueue(list_msg.my_list, buffer);
 					agent_count++;
-					if (agent_count > handle_agent_count)
+					if (agent_count > 1000)
 						break;
 				}
 				list_iterator_destroy(agent_itr);
 				buffer = pack_slurmdbd_msg(
 					&list_req, SLURM_PROTOCOL_VERSION);
-			} else if (cnt > 1) {
-				list_msg.my_list = agent_list;
-				buffer = pack_slurmdbd_msg(
-					&list_req, SLURM_PROTOCOL_VERSION);
 			} else
-				buffer = (Buf) list_peek(agent_list);
+				buffer = list_peek(agent_list);
 		} else
 			buffer = NULL;
 		slurm_mutex_unlock(&agent_lock);
@@ -753,9 +727,9 @@ static void *_agent(void *x)
 					FREE_NULL_LIST(list_msg.my_list);
 				list_msg.my_list = NULL;
 			} else
-				buffer = (Buf) list_dequeue(agent_list);
+				buffer = list_dequeue(agent_list);
 
-			free_buf(buffer);
+			FREE_NULL_BUFFER(buffer);
 			fail_time = 0;
 		} else {
 			/* We need to free a mult_msg even on failure */
@@ -763,7 +737,7 @@ static void *_agent(void *x)
 				if (list_msg.my_list != agent_list)
 					FREE_NULL_LIST(list_msg.my_list);
 				list_msg.my_list = NULL;
-				free_buf(buffer);
+				FREE_NULL_BUFFER(buffer);
 			}
 
 			fail_time = time(NULL);
@@ -810,39 +784,35 @@ static void _create_agent(void)
 
 static void _shutdown_agent(void)
 {
-	int i;
+	if (!agent_tid)
+		return;
 
-	if (agent_tid) {
-		slurmdbd_shutdown = time(NULL);
-		for (i=0; i<50; i++) {	/* up to 5 secs total */
-			slurm_mutex_lock(&agent_lock);
-			if (!agent_running) {
-				slurm_mutex_unlock(&agent_lock);
-				goto fini;
-			}
-			slurm_cond_broadcast(&agent_cond);
+	slurmdbd_shutdown = time(NULL);
+	for (int i = 0; i < 50; i++) {	/* up to 5 secs total */
+		slurm_mutex_lock(&agent_lock);
+		if (!agent_running) {
 			slurm_mutex_unlock(&agent_lock);
-
-			usleep(100000);	/* 0.1 sec per try */
-			if (pthread_kill(agent_tid, SIGUSR1))
-				break;
-
+			goto fini;
 		}
+		slurm_cond_broadcast(&agent_cond);
+		slurm_mutex_unlock(&agent_lock);
 
-		/* On rare occasions agent thread may not end quickly,
-		 * perhaps due to communication problems with slurmdbd.
-		 * Cancel it and join before returning or we could remove
-		 * and leave the agent without valid data */
-		if (pthread_kill(agent_tid, 0) == 0) {
-			error("agent failed to shutdown gracefully");
-			error("unable to save pending requests");
-			pthread_cancel(agent_tid);
-		}
+		usleep(100000);	/* 0.1 sec per try */
+	}
+
+	/*
+	 * On rare occasions agent thread may not end quickly,
+	 * perhaps due to communication problems with slurmdbd.
+	 * Cancel it and join before returning or we could remove
+	 * and leave the agent without valid data.
+	 */
+	error("agent failed to shutdown gracefully");
+	error("unable to save pending requests");
+	pthread_cancel(agent_tid);
 
 fini:
-		pthread_join(agent_tid,  NULL);
-		agent_tid = 0;
-	}
+	pthread_join(agent_tid,  NULL);
+	agent_tid = 0;
 }
 
 /****************************************************************************
@@ -923,7 +893,7 @@ extern int slurmdbd_agent_send_recv(uint16_t rpc_version,
  * Returns SLURM_SUCCESS or an error code */
 extern int slurmdbd_agent_send(uint16_t rpc_version, persist_msg_t *req)
 {
-	Buf buffer;
+	buf_t *buffer;
 	uint32_t cnt, rc = SLURM_SUCCESS;
 	static time_t syslog_time = 0;
 
@@ -944,7 +914,7 @@ extern int slurmdbd_agent_send(uint16_t rpc_version, persist_msg_t *req)
 		_create_agent();
 		if ((agent_tid == 0) || (agent_list == NULL)) {
 			slurm_mutex_unlock(&agent_lock);
-			free_buf(buffer);
+			FREE_NULL_BUFFER(buffer);
 			return SLURM_ERROR;
 		}
 	}
@@ -963,15 +933,14 @@ extern int slurmdbd_agent_send(uint16_t rpc_version, persist_msg_t *req)
 	_max_dbd_msg_action(&cnt);
 
 	if (cnt < slurm_conf.max_dbd_msgs) {
-		if (list_enqueue(agent_list, buffer) == NULL)
-			fatal("list_enqueue: memory allocation failure");
+		list_enqueue(agent_list, buffer);
 	} else {
 		error("agent queue is full (%u), discarding %s:%u request",
 		      cnt,
 		      slurmdbd_msg_type_2_str(req->msg_type, 1),
 		      req->msg_type);
 		(slurmdbd_conn->trigger_callbacks.acct_full)();
-		free_buf(buffer);
+		FREE_NULL_BUFFER(buffer);
 		rc = SLURM_ERROR;
 	}
 
