@@ -38,21 +38,18 @@
 
 #define _GNU_SOURCE
 
-#include "src/common/slurm_xlator.h"
-#include "src/common/gres.h"
-#include "src/common/log.h"
-#include "src/common/read_config.h"
+#include <dlfcn.h>
 #include <rocm_smi/rocm_smi.h>
 
+#include "../common/gpu_common.h"
+
+#ifdef HAVE_NUMA
+#  include <numa.h>
+#endif
+
 /*
  * #defines needed to test rsmi.
  */
-#define FREQS_CONCISE	5 // This must never be smaller than 5, or error
-
-#define GPU_LOW		((unsigned int) -1)
-#define GPU_MEDIUM	((unsigned int) -2)
-#define GPU_HIGH_M1	((unsigned int) -3)
-#define GPU_HIGH	((unsigned int) -4)
 
 static bitstr_t	*saved_gpus;
 
@@ -68,17 +65,17 @@ typedef struct rsmiPciInfo_st {
 	union {
 		struct {
 #ifdef SLURM_BIGENDIAN
-			uint64_t reserved : 35;
-			uint64_t domain : 16;
-			uint64_t bus : 5;
+			uint64_t domain : 32;
+			uint64_t reserved : 16;
+			uint64_t bus : 8;
 			uint64_t device : 5;
 			uint64_t function : 3;
 #else
 			uint64_t function : 3;
 			uint64_t device : 5;
-			uint64_t bus : 5;
-			uint64_t domain : 16;
-			uint64_t reserved : 35;
+			uint64_t bus : 8;
+			uint64_t reserved : 16;
+			uint64_t domain : 32;
 #endif
 		};
 		uint64_t bdfid;
@@ -110,12 +107,21 @@ typedef struct rsmiPciInfo_st {
  * plugin_version - an unsigned 32-bit integer containing the Slurm version
  * (major.minor.micro combined into a single number).
  */
-const char	*plugin_name		= "GPU RSMI plugin";
+const char plugin_name[] = "GPU RSMI plugin";
 const char	plugin_type[]		= "gpu/rsmi";
 const uint32_t	plugin_version		= SLURM_VERSION_NUMBER;
 
+static int gpumem_pos = -1;
+static int gpuutil_pos = -1;
+
 extern int init(void)
 {
+	rsmi_init(0);
+
+	if (running_in_slurmstepd()) {
+		gpu_get_tres_pos(&gpumem_pos, &gpuutil_pos);
+	}
+
 	debug("%s: %s loaded", __func__, plugin_name);
 
 	return SLURM_SUCCESS;
@@ -125,127 +131,11 @@ extern int fini(void)
 {
 	debug("%s: unloading %s", __func__, plugin_name);
 
+	rsmi_shut_down();
+
 	return SLURM_SUCCESS;
 }
 
-//TODO: Duplicated from NVML plugin. Move to a common directory
-static unsigned int _xlate_freq_value(char *gpu_freq)
-{
-	unsigned int value;
-
-	if (!gpu_freq && (gpu_freq[0] < '0') && (gpu_freq[0] > '9'))
-		return 0;	/* Not a numeric value */
-	value = strtoul(gpu_freq, NULL, 10);
-	return value;
-}
-
-//TODO: Duplicated from NVML plugin. Move to a common directory
-static unsigned int _xlate_freq_code(char *gpu_freq)
-{
-	//TODO: To be moved to common directory
-	if (!gpu_freq || !gpu_freq[0])
-		return 0;
-	if ((gpu_freq[0] >= '0') && (gpu_freq[0] <= '9'))
-		return 0;	/* Pure numeric value */
-	if (!strcasecmp(gpu_freq, "low"))
-		return GPU_LOW;
-	else if (!strcasecmp(gpu_freq, "medium"))
-		return GPU_MEDIUM;
-	else if (!strcasecmp(gpu_freq, "highm1"))
-		return GPU_HIGH_M1;
-	else if (!strcasecmp(gpu_freq, "high"))
-		return GPU_HIGH;
-
-	debug("%s: %s: Invalid job GPU frequency (%s)",
-	      plugin_type, __func__, gpu_freq);
-	return 0;	/* Bad user input */
-}
-
-//TODO: Duplicated from NVML plugin. Move to a common directory
-static void _parse_gpu_freq2(char *gpu_freq, unsigned int *gpu_freq_code,
-			     unsigned int *gpu_freq_value,
-			     unsigned int *mem_freq_code,
-			     unsigned int *mem_freq_value, bool *verbose_flag)
-{
-	char *tmp, *tok, *sep, *save_ptr = NULL;
-
-	if (!gpu_freq || !gpu_freq[0])
-		return;
-	tmp = xstrdup(gpu_freq);
-	tok = strtok_r(tmp, ",", &save_ptr);
-	while (tok) {
-		sep = strchr(tok, '=');
-		if (sep) {
-			sep[0] = '\0';
-			sep++;
-			if (!strcasecmp(tok, "memory")) {
-				*mem_freq_code = _xlate_freq_code(sep);
-				*mem_freq_value = _xlate_freq_value(sep);
-				if (!(*mem_freq_code) && !(*mem_freq_value)) {
-					debug("Invalid job GPU memory frequency: %s",
-					      tok);
-				}
-			} else {
-				debug("%s: %s: Invalid job device frequency type: %s",
-				      plugin_type, __func__, tok);
-			}
-		} else if (!strcasecmp(tok, "verbose")) {
-			*verbose_flag = true;
-		} else {
-			*gpu_freq_code = _xlate_freq_code(tok);
-			*gpu_freq_value = _xlate_freq_value(tok);
-			if (!(*gpu_freq_code) && !(*gpu_freq_value))
-				debug("Invalid job GPU frequency: %s", tok);
-		}
-		tok = strtok_r(NULL, ",", &save_ptr);
-	}
-	xfree(tmp);
-}
-
-//TODO: Duplicated from NVML plugin. Move to a common directory
-static void _parse_gpu_freq(char *gpu_freq, unsigned int *gpu_freq_num,
-			    unsigned int *mem_freq_num, bool *verbose_flag)
-{
-	unsigned int def_gpu_freq_code = 0, def_gpu_freq_value = 0;
-	unsigned int def_mem_freq_code = 0, def_mem_freq_value = 0;
-	unsigned int job_gpu_freq_code = 0, job_gpu_freq_value = 0;
-	unsigned int job_mem_freq_code = 0, job_mem_freq_value = 0;
-	char *def_freq;
-
-	_parse_gpu_freq2(gpu_freq, &job_gpu_freq_code, &job_gpu_freq_value,
-			 &job_mem_freq_code, &job_mem_freq_value, verbose_flag);
-
-	// Defaults to high for both mem and gfx
-	def_freq = slurm_get_gpu_freq_def();
-	_parse_gpu_freq2(def_freq, &def_gpu_freq_code, &def_gpu_freq_value,
-			 &def_mem_freq_code, &def_mem_freq_value, verbose_flag);
-	xfree(def_freq);
-
-	if (job_gpu_freq_code)
-		*gpu_freq_num = job_gpu_freq_code;
-	else if (job_gpu_freq_value)
-		*gpu_freq_num = job_gpu_freq_value;
-	else if (def_gpu_freq_code)
-		*gpu_freq_num = def_gpu_freq_code;
-	else if (def_gpu_freq_value)
-		*gpu_freq_num = def_gpu_freq_value;
-
-	if (job_mem_freq_code)
-		*mem_freq_num = job_mem_freq_code;
-	else if (job_mem_freq_value)
-		*mem_freq_num = job_mem_freq_value;
-	else if (def_mem_freq_code)
-		*mem_freq_num = def_mem_freq_code;
-	else if (def_mem_freq_value)
-		*mem_freq_num = def_mem_freq_value;
-}
-
-//TODO: Duplicated from NVML plugin. Move to a common directory
-static int _sort_freq_descending(const void *a, const void *b)
-{
-	return (*(unsigned long *)b - *(unsigned long *)a);
-}
-
 /*
  * Get all possible memory frequencies for the device
  *
@@ -338,169 +228,33 @@ static void _rsmi_print_freqs(uint32_t dv_ind, log_level_t l)
 	unsigned int mem_freqs[RSMI_MAX_NUM_FREQUENCIES] = {0};
 	unsigned int gfx_freqs[RSMI_MAX_NUM_FREQUENCIES] = {0};
 	unsigned int size = RSMI_MAX_NUM_FREQUENCIES;
-	bool concise = false;
-	unsigned int i;
 
 	if (!_rsmi_get_mem_freqs(dv_ind, &size, mem_freqs))
 		return;
 
-	qsort(mem_freqs, size,
-	      sizeof(unsigned int), _sort_freq_descending);
+	qsort(mem_freqs, size, sizeof(unsigned int),
+	      gpu_common_sort_freq_descending);
 	if ((size > 1) && (mem_freqs[0] <= mem_freqs[(size)-1])) {
 		error("%s: memory frequencies are not stored in descending order!",
 		      __func__);
 		return;
 	}
 
-	if (size > FREQS_CONCISE)
-		concise = true;
-
-	log_var(l, "        Possible GPU Memory Frequencies (%u):", size);
-	log_var(l, "        ---------------------------------");
-	if (!concise) {
-		for (i = 0; i < size; ++i)
-			log_var(l, "          *%u MHz [%u]", mem_freqs[i], i);
-	} else {
-		// first, next, ..., middle, ..., penultimate, last
-		log_var(l, "          *%u MHz [0]", mem_freqs[0]);
-		log_var(l, "          *%u MHz [1]", mem_freqs[1]);
-		log_var(l, "          ...");
-		log_var(l, "          *%u MHz [%u]", mem_freqs[(size - 1) / 2],
-			(size - 1) / 2);
-		log_var(l, "          ...");
-		log_var(l, "          *%u MHz [%u]",
-			mem_freqs[size - 2], size - 2);
-		log_var(l, "          *%u MHz [%u]",
-			mem_freqs[size - 1], size - 1);
-	}
+	gpu_common_print_freqs(mem_freqs, size, l, "GPU Memory", 0);
 
 	size = RSMI_MAX_NUM_FREQUENCIES;
 	if (!_rsmi_get_gfx_freqs(dv_ind, &size, gfx_freqs))
 		return;
 
-	qsort(gfx_freqs, size,
-	      sizeof(unsigned int), _sort_freq_descending);
+	qsort(gfx_freqs, size, sizeof(unsigned int),
+	      gpu_common_sort_freq_descending);
 	if ((size > 1) && (gfx_freqs[0] <= gfx_freqs[(size)-1])) {
 		error("%s: Graphics frequencies are not stored in descending order!",
 		      __func__);
 		return;
 	}
 
-	if (size > FREQS_CONCISE)
-		concise = true;
-
-	log_var(l, "        Possible GPU Graphics Frequencies (%u):", size);
-	log_var(l, "        ---------------------------------");
-	if (!concise) {
-		for (i = 0; i < size; ++i)
-			log_var(l, "          *%u MHz [%u]", gfx_freqs[i], i);
-		return;
-	}
-	// first, next, ..., middle, ..., penultimate, last
-	log_var(l, "          *%u MHz [0]", gfx_freqs[0]);
-	log_var(l, "          *%u MHz [1]", gfx_freqs[1]);
-	log_var(l, "          ...");
-	log_var(l, "          *%u MHz [%u]", gfx_freqs[(size - 1) / 2],
-		(size - 1) / 2);
-	log_var(l, "          ...");
-	log_var(l, "          *%u MHz [%u]", gfx_freqs[size - 2], size - 2);
-	log_var(l, "          *%u MHz [%u]", gfx_freqs[size - 1], size - 1);
-}
-
-/*
- * Convert frequency to nearest valid frequency found in frequency array
- *
- * freq		(IN/OUT) The frequency to check, in MHz. Also the output, if
- *		it needs to be changed.
- * freqs_size	(IN) The size of the freqs array
- * freqs	(IN) An array of frequency values in MHz, sorted highest to
- *		lowest
- *
- * Inspired by src/common/cpu_frequency#_cpu_freq_freqspec_num()
- */
-//TODO: Duplicated from NVML plugin. Move to a common directory
-static void _get_nearest_freq(unsigned int *freq, unsigned int freqs_size,
-			      unsigned int *freqs)
-{
-	unsigned int i;
-
-	if (!freq || !(*freq)) {
-		log_flag(GRES, "%s: No frequency supplied", __func__);
-		return;
-	}
-	if (!freqs || !(*freqs)) {
-		log_flag(GRES, "%s: No frequency list supplied", __func__);
-		return;
-	}
-	if (freqs_size <= 0) {
-		log_flag(GRES, "%s: Frequency list is empty", __func__);
-		return;
-	}
-
-	// Check for special case values; freqs is sorted in descending order
-	switch ((*freq)) {
-	case GPU_LOW:
-		*freq = freqs[freqs_size - 1];
-		debug2("Frequency GPU_LOW: %u MHz", *freq);
-		return;
-
-	case GPU_MEDIUM:
-		*freq = freqs[(freqs_size - 1) / 2];
-		debug2("Frequency GPU_MEDIUM: %u MHz", *freq);
-		return;
-
-	case GPU_HIGH_M1:
-		if (freqs_size == 1)
-			*freq = freqs[0];
-		else
-			*freq = freqs[1];
-		debug2("Frequency GPU_HIGH_M1: %u MHz", *freq);
-		return;
-
-	case GPU_HIGH:
-		*freq = freqs[0];
-		debug2("Frequency GPU_HIGH: %u MHz", *freq);
-		return;
-
-	default:
-		debug2("Freq is not a special case. Continue...");
-		break;
-	}
-
-	/* check if freq is out of bounds of freqs */
-	if (*freq > freqs[0]) {
-		log_flag(GRES, "Rounding frequency %u MHz down to %u MHz",
-		         *freq, freqs[0]);
-		*freq = freqs[0];
-		return;
-	} else if (*freq < freqs[freqs_size - 1]) {
-		log_flag(GRES, "Rounding frequency %u MHz up to %u MHz",
-		         *freq, freqs[freqs_size - 1]);
-		*freq = freqs[freqs_size - 1];
-		return;
-	}
-
-	/* check for frequency, and round up if no exact match */
-	for (i = 0; i < freqs_size - 1;) {
-		if (*freq == freqs[i]) {
-			// No change necessary
-			debug2("No change necessary. Freq: %u MHz", *freq);
-			return;
-		}
-		i++;
-		/*
-		 * Step down to next element to round up.
-		 * Safe to advance due to bounds checks above here
-		 */
-		if (*freq > freqs[i]) {
-			log_flag(GRES, "Rounding frequency %u MHz up to %u MHz",
-			         *freq, freqs[i - 1]);
-			*freq = freqs[i - 1];
-			return;
-		}
-	}
-	error("%s: Got to the end of the function. Freq: %u MHz",
-	      __func__, *freq);
+	gpu_common_print_freqs(gfx_freqs, size, l, "GPU Graphics", 0);
 }
 
 /*
@@ -532,8 +286,8 @@ static void _rsmi_get_nearest_freqs(uint32_t dv_ind,
 		return;
 
 	memcpy(mem_freqs_sort, mem_freqs, mem_freqs_size*sizeof(unsigned int));
-	qsort(mem_freqs_sort, mem_freqs_size,
-	      sizeof(unsigned int), _sort_freq_descending);
+	qsort(mem_freqs_sort, mem_freqs_size, sizeof(unsigned int),
+	      gpu_common_sort_freq_descending);
 	if ((mem_freqs_size > 1) &&
 	    (mem_freqs_sort[0] <= mem_freqs_sort[(mem_freqs_size)-1])) {
 		error("%s: memory frequencies are not stored in descending order!",
@@ -542,10 +296,10 @@ static void _rsmi_get_nearest_freqs(uint32_t dv_ind,
 	}
 
 	// Set the nearest valid memory frequency for the requested frequency
-	_get_nearest_freq(mem_freq, mem_freqs_size, mem_freqs_sort);
+	gpu_common_get_nearest_freq(mem_freq, mem_freqs_size, mem_freqs_sort);
 
 	// convert the frequency to bit mask
-	for (int i = 0; i < mem_freqs_size; i++)
+	for (uint64_t i = 0; i < mem_freqs_size; i++)
 		if (*mem_freq == mem_freqs[i]) {
 			*mem_bitmask = (1 << i);
 			break;
@@ -556,8 +310,8 @@ static void _rsmi_get_nearest_freqs(uint32_t dv_ind,
 		return;
 
 	memcpy(gfx_freqs_sort, gfx_freqs, gfx_freqs_size*sizeof(unsigned int));
-	qsort(gfx_freqs_sort, gfx_freqs_size,
-	      sizeof(unsigned int), _sort_freq_descending);
+	qsort(gfx_freqs_sort, gfx_freqs_size, sizeof(unsigned int),
+	      gpu_common_sort_freq_descending);
 	if ((gfx_freqs_size > 1) &&
 	    (gfx_freqs_sort[0] <= gfx_freqs_sort[(gfx_freqs_size)-1])) {
 		error("%s: graphics frequencies are not stored in descending order!",
@@ -566,10 +320,10 @@ static void _rsmi_get_nearest_freqs(uint32_t dv_ind,
 	}
 
 	// Set the nearest valid graphics frequency for the requested frequency
-	_get_nearest_freq(gfx_freq, gfx_freqs_size, gfx_freqs_sort);
+	gpu_common_get_nearest_freq(gfx_freq, gfx_freqs_size, gfx_freqs_sort);
 
 	// convert the frequency to bit mask
-	for (int i = 0; i < gfx_freqs_size; i++)
+	for (uint64_t i = 0; i < gfx_freqs_size; i++)
 		if (*gfx_freq == gfx_freqs[i]) {
 			*gfx_bitmask = (1 << i);
 			break;
@@ -705,27 +459,6 @@ static unsigned int _rsmi_get_mem_freq(uint32_t dv_ind)
 	return _rsmi_get_freq(dv_ind, RSMI_CLK_TYPE_MEM);
 }
 
-/*
- * Convert a frequency value to a string
- * Returned string must be xfree()'ed
- */
-//TODO: Duplicated from NVML plugin. Move to a common directory
-static char *_freq_value_to_string(unsigned int freq)
-{
-	switch (freq) {
-	case GPU_LOW:
-		return xstrdup("low");
-	case GPU_MEDIUM:
-		return xstrdup("medium");
-	case GPU_HIGH:
-		return xstrdup("high");
-	case GPU_HIGH_M1:
-		return xstrdup("highm1");
-	default:
-		return xstrdup_printf("%u", freq);
-	}
-}
-
 /*
  * Reset the frequencies of each GPU in the step to the hardware default
  * NOTE: RSMI must be initialized beforehand
@@ -754,8 +487,6 @@ static void _reset_freq(bitstr_t *gpus)
 		debug2("Graphics frequency after reset: %u",
 		       _rsmi_get_gfx_freq(i));
 
-		// TODO: Check to make sure that the frequency reset
-
 		if (freq_reset) {
 			log_flag(GRES, "Successfully reset GPU[%d]", i);
 			count_set++;
@@ -788,24 +519,23 @@ static void _set_freq(bitstr_t *gpus, char *gpu_freq)
 	int gpu_len = 0;
 	int i = -1, count = 0, count_set = 0;
 	unsigned int gpu_freq_num = 0, mem_freq_num = 0;
-	uint64_t mem_bitmask = 0, gpu_bitmask = 0;
 	bool freq_set = false, freq_logged = false;
 	char *tmp = NULL;
-	slurm_cgroup_conf_t *cg_conf;
 	bool task_cgroup = false;
 	bool constrained_devices = false;
 	bool cgroups_active = false;
 
 	// Parse frequency information
 	debug2("_parse_gpu_freq(%s)", gpu_freq);
-	_parse_gpu_freq(gpu_freq, &gpu_freq_num, &mem_freq_num, &verbose_flag);
+	gpu_common_parse_gpu_freq(gpu_freq, &gpu_freq_num, &mem_freq_num,
+				  &verbose_flag);
 	if (verbose_flag)
 		debug2("verbose_flag ON");
 
-	tmp = _freq_value_to_string(mem_freq_num);
+	tmp = gpu_common_freq_value_to_string(mem_freq_num);
 	debug2("Requested GPU memory frequency: %s", tmp);
 	xfree(tmp);
-	tmp = _freq_value_to_string(gpu_freq_num);
+	tmp = gpu_common_freq_value_to_string(gpu_freq_num);
 	debug2("Requested GPU graphics frequency: %s", tmp);
 	xfree(tmp);
 
@@ -815,11 +545,9 @@ static void _set_freq(bitstr_t *gpus, char *gpu_freq)
 	}
 
 	// Check if GPUs are constrained by cgroups
-	slurm_mutex_lock(&xcgroup_config_read_mutex);
-	cg_conf = xcgroup_get_slurm_cgroup_conf();
-	if (cg_conf && cg_conf->constrain_devices)
+	cgroup_conf_init();
+	if (slurm_cgroup_conf.constrain_devices)
 		constrained_devices = true;
-	slurm_mutex_unlock(&xcgroup_config_read_mutex);
 
 	// Check if task/cgroup plugin is loaded
 	if (xstrstr(slurm_conf.task_plugin, "cgroup"))
@@ -840,6 +568,8 @@ static void _set_freq(bitstr_t *gpus, char *gpu_freq)
 	// Set the frequency of each device allocated to the step
 	for (i = 0; i < gpu_len; i++) {
 		char *sep = "";
+		uint64_t mem_bitmask = 0, gpu_bitmask = 0;
+		unsigned int gpu_freq = gpu_freq_num, mem_freq = mem_freq_num;
 
 		// Only check the global GPU bitstring if not using cgroups
 		if (!cgroups_active && !bit_test(gpus, i)) {
@@ -849,8 +579,8 @@ static void _set_freq(bitstr_t *gpus, char *gpu_freq)
 		count++;
 
 		debug2("Setting frequency of RSMI device %u", i);
-		_rsmi_get_nearest_freqs(i, &mem_freq_num, &mem_bitmask,
-					&gpu_freq_num, &gpu_bitmask);
+		_rsmi_get_nearest_freqs(i, &mem_freq, &mem_bitmask,
+					&gpu_freq, &gpu_bitmask);
 
 		debug2("Memory frequency before set: %u",
 		       _rsmi_get_mem_freq(i));
@@ -862,13 +592,12 @@ static void _set_freq(bitstr_t *gpus, char *gpu_freq)
 		debug2("Graphics frequency after set: %u",
 		       _rsmi_get_gfx_freq(i));
 
-		if (mem_freq_num) {
-			xstrfmtcat(tmp, "%smemory_freq:%u", sep, mem_freq_num);
+		if (mem_freq) {
+			xstrfmtcat(tmp, "%smemory_freq:%u", sep, mem_freq);
 			sep = ",";
 		}
-		if (gpu_freq_num) {
-			xstrfmtcat(tmp, "%sgraphics_freq:%u", sep,
-				   gpu_freq_num);
+		if (gpu_freq) {
+			xstrfmtcat(tmp, "%sgraphics_freq:%u", sep, gpu_freq);
 		}
 
 		if (freq_set) {
@@ -930,7 +659,7 @@ static void _rsmi_get_version(char *version, unsigned int len)
  *
  * device_count	(OUT) Number of available GPU devices
  */
-static void _rsmi_get_device_count(unsigned int *device_count)
+extern void gpu_p_get_device_count(unsigned int *device_count)
 {
 	const char *status_string;
 	rsmi_status_t rsmi_rc = rsmi_num_monitor_devices(device_count);
@@ -959,6 +688,7 @@ static void _rsmi_get_device_name(uint32_t dv_ind, char *device_name,
 		rsmi_rc = rsmi_status_string(rsmi_rc, &status_string);
 		error("RSMI: Failed to get name of the GPU: %s", status_string);
 	}
+	gpu_common_underscorify_tolower(device_name);
 }
 
 /*
@@ -1037,6 +767,56 @@ static void _rsmi_get_device_unique_id(uint32_t dv_ind, uint64_t *id)
 	}
 }
 
+static bitstr_t *_rsmi_get_device_cpu_mask(uint32_t dv_ind)
+{
+	bitstr_t *cpu_aff_mac_bitstr = NULL;
+#ifdef HAVE_NUMA
+	uint32_t nnid = 1;
+	uint16_t maxcpus = conf->sockets * conf->cores * conf->threads;
+	struct bitmask *collective;
+	rsmi_status_t rsmi_rc =	rsmi_topo_get_numa_node_number(dv_ind, &nnid);
+
+	if (rsmi_rc != RSMI_STATUS_SUCCESS) {
+		const char *status_string;
+		rsmi_rc = rsmi_status_string(rsmi_rc, &status_string);
+		error("RSMI: Failed to get numa affinity of the GPU: %s",
+		      status_string);
+		return NULL;
+	}
+
+	collective = numa_allocate_cpumask();
+	if (maxcpus > collective->size) {
+		error("Size mismatch!!!! %d %lu", maxcpus, collective->size);
+		numa_free_cpumask(collective);
+		return NULL;
+	}
+
+	/*
+	 * FIXME: This is a hack (copied from task/affinity/numa.c to make it
+	 * work like NUMA v2, but for the time being we are stuck on
+	 * v1. (numa_node_to_cpus will multiple the size by 8 and the collective
+	 * is already at the correct size)
+	 */
+	if (numa_node_to_cpus(nnid, collective->maskp, collective->size / 8)) {
+		error("numa_node_to_cpus: %m");
+		numa_free_cpumask(collective);
+		return NULL;
+	}
+
+	/* Convert the collective to a slurm bitstr_t */
+	cpu_aff_mac_bitstr = bit_alloc(maxcpus);
+	for (int i = 0; i < maxcpus; i++) {
+		if (!numa_bitmask_isbitset(collective, i))
+			continue;
+
+		bit_set(cpu_aff_mac_bitstr, i);
+	}
+
+	numa_free_cpumask(collective);
+#endif
+	return cpu_aff_mac_bitstr;
+}
+
 /*
  * Creates and returns a gres conf list of detected AMD gpus on the node.
  * If an error occurs, return NULL
@@ -1055,24 +835,47 @@ static List _get_system_gpu_list_rsmi(node_config_load_t *node_config)
 	char driver[RSMI_STRING_BUFFER_SIZE];
 	char version[RSMI_STRING_BUFFER_SIZE];
 
-	rsmi_init(0);
-
 	_rsmi_get_driver(driver, RSMI_STRING_BUFFER_SIZE);
 	_rsmi_get_version(version, RSMI_STRING_BUFFER_SIZE);
 	debug("AMD Graphics Driver Version: %s", driver);
 	debug("RSMI Library Version: %s", version);
 
-	_rsmi_get_device_count(&device_count);
+	gpu_p_get_device_count(&device_count);
 	debug2("Device count: %d", device_count);
 
 	// Loop through all the GPUs on the system and add to gres_list_system
 	for (i = 0; i < device_count; ++i) {
 		unsigned int minor_number = 0;
-		char *device_file = NULL;
 		char device_name[RSMI_STRING_BUFFER_SIZE] = {0};
 		char device_brand[RSMI_STRING_BUFFER_SIZE] = {0};
 		rsmiPciInfo_t pci_info;
 		uint64_t uuid = 0;
+		char *cpu_aff_mac_range = NULL;
+		gres_slurmd_conf_t gres_slurmd_conf = {
+			.config_flags = GRES_CONF_ENV_RSMI,
+			.count = 1,
+			.cpu_cnt = node_config->cpu_cnt,
+			.cpus_bitmap = _rsmi_get_device_cpu_mask(i),
+			.name = "gpu",
+		};
+
+		if (gres_slurmd_conf.cpus_bitmap) {
+			cpu_aff_mac_range = bit_fmt_full(
+				gres_slurmd_conf.cpus_bitmap);
+
+			/*
+			 * Convert cpu range str from machine to abstract(slurm)
+			 * format
+			 */
+			if (node_config->xcpuinfo_mac_to_abs(
+				    cpu_aff_mac_range,
+				    &gres_slurmd_conf.cpus)) {
+				error("Conversion from machine to abstract failed");
+				FREE_NULL_BITMAP(gres_slurmd_conf.cpus_bitmap);
+				xfree(cpu_aff_mac_range);
+				continue;
+			}
+		}
 
 		_rsmi_get_device_name(i, device_name, RSMI_STRING_BUFFER_SIZE);
 		_rsmi_get_device_brand(i, device_brand,
@@ -1082,7 +885,12 @@ static List _get_system_gpu_list_rsmi(node_config_load_t *node_config)
 		_rsmi_get_device_pci_info(i, &pci_info);
 		_rsmi_get_device_unique_id(i, &uuid);
 
-		xstrfmtcat(device_file, "/dev/dri/renderD%u", minor_number);
+		/* Use links to record PCI bus ID order */
+		gres_slurmd_conf.links =
+			gres_links_create_empty(i, device_count);
+
+		xstrfmtcat(gres_slurmd_conf.file,
+			   "/dev/dri/renderD%u", minor_number);
 
 		debug2("GPU index %u:", i);
 		debug2("    Name: %s", device_name);
@@ -1091,23 +899,31 @@ static List _get_system_gpu_list_rsmi(node_config_load_t *node_config)
 		debug2("    PCI Domain/Bus/Device/Function: %u:%u:%u.%u",
 		       pci_info.domain,
 		       pci_info.bus, pci_info.device, pci_info.function);
-		debug2("    Device File (minor number): %s", device_file);
+		debug2("    Links: %s", gres_slurmd_conf.links);
+		debug2("    Device File (minor number): %s",
+			gres_slurmd_conf.file);
 		if (minor_number != i+128)
 			debug("Note: GPU index %u is different from minor # %u",
 			      i, minor_number);
+		debug2("    CPU Affinity Range - Machine: %s",
+		       cpu_aff_mac_range);
+		debug2("    Core Affinity Range - Abstract: %s",
+		       gres_slurmd_conf.cpus);
 
 		// Print out possible memory frequencies for this device
 		_rsmi_print_freqs(i, LOG_LEVEL_DEBUG2);
 
-		add_gres_to_list(gres_list_system, "gpu", 1,
-				 node_config->cpu_cnt, NULL, NULL,
-				 device_file, device_brand, NULL);
+		gres_slurmd_conf.type_name = device_brand;
 
-		xfree(device_file);
+		add_gres_to_list(gres_list_system, &gres_slurmd_conf);
+
+		FREE_NULL_BITMAP(gres_slurmd_conf.cpus_bitmap);
+		xfree(cpu_aff_mac_range);
+		xfree(gres_slurmd_conf.cpus);
+		xfree(gres_slurmd_conf.file);
+		xfree(gres_slurmd_conf.links);
 	}
 
-	rsmi_shut_down();
-
 	info("%u GPU system device(s) detected", device_count);
 	return gres_list_system;
 }
@@ -1153,8 +969,6 @@ extern void gpu_p_step_hardware_init(bitstr_t *usable_gpus, char *tres_freq)
 	FREE_NULL_BITMAP(saved_gpus);
 	saved_gpus = bit_copy(usable_gpus);
 
-	rsmi_init(0);
-
 	// Set the frequency of each GPU index specified in the bitstr
 	_set_freq(usable_gpus, freq);
 	xfree(freq);
@@ -1176,3 +990,65 @@ extern char *gpu_p_test_cpu_conv(char *cpu_range)
 {
 	return NULL;
 }
+
+/*
+ * gpu_p_energy_read read current average watts and update last_update_watt
+ *
+ * dv_ind         (IN) The device index
+ * energy         (IN) A pointer to gpu_status_t structure
+ */
+extern int gpu_p_energy_read(uint32_t dv_ind, gpu_status_t *gpu)
+{
+	const char *status_string;
+	uint64_t curr_milli_watts;
+	rsmi_status_t rsmi_rc = rsmi_dev_power_ave_get(
+		dv_ind, 0, &curr_milli_watts);
+
+	if (rsmi_rc != RSMI_STATUS_SUCCESS) {
+		rsmi_rc = rsmi_status_string(rsmi_rc, &status_string);
+		error("RSMI: Failed to get power: %s", status_string);
+		gpu->energy.current_watts = NO_VAL;
+		return SLURM_ERROR;
+	}
+
+	gpu->last_update_watt = curr_milli_watts/1000000;
+	gpu->previous_update_time = gpu->last_update_time;
+	gpu->last_update_time = time(NULL);
+
+	return SLURM_SUCCESS;
+}
+
+extern int gpu_p_usage_read(pid_t pid, acct_gather_data_t *data)
+{
+	const char *status_string;
+	rsmi_process_info_t proc = {0};
+	rsmi_status_t rc;
+
+	if ((gpuutil_pos == -1) || (gpumem_pos == -1)) {
+		debug2("%s: We are not tracking TRES gpuutil/gpumem", __func__);
+		return SLURM_SUCCESS;
+	}
+
+	rc = rsmi_compute_process_info_by_pid_get(pid, &proc);
+
+	if (rc == RSMI_STATUS_NOT_FOUND) {
+		debug2("Couldn't find pid %d, probably hasn't started yet or has already finished",
+		       pid);
+		return SLURM_SUCCESS;
+	} else if (rc != RSMI_STATUS_SUCCESS) {
+		(void) rsmi_status_string(rc, &status_string);
+		error("RSMI: Failed to get usage(%d): %s", rc, status_string);
+		return SLURM_ERROR;
+	}
+
+	data[gpuutil_pos].size_read = proc.cu_occupancy;
+
+	data[gpumem_pos].size_read = proc.vram_usage;
+
+	log_flag(JAG, "pid %d has GPUUtil=%lu and MemMB=%lu",
+		 pid,
+		 data[gpuutil_pos].size_read,
+		 data[gpumem_pos].size_read / 1048576);
+
+	return SLURM_SUCCESS;
+}
