@@ -56,6 +56,7 @@
 
 #include "src/common/slurm_xlator.h"
 #include "src/common/assoc_mgr.h"
+#include "src/interfaces/gres.h"
 #include "src/common/uid.h"
 #include "src/lua/slurm_lua.h"
 #include "src/slurmctld/locks.h"
@@ -94,14 +95,17 @@ const char plugin_name[]       	= "Job submit lua plugin";
 const char plugin_type[]       	= "job_submit/lua";
 const uint32_t plugin_version   = SLURM_VERSION_NUMBER;
 
-static const char lua_script_path[] = DEFAULT_SCRIPT_DIR "/job_submit.lua";
+static char *lua_script_path;
 static time_t lua_script_last_loaded = (time_t) 0;
 static lua_State *L = NULL;
 static char *user_msg = NULL;
-
 time_t last_lua_jobs_update = (time_t) 0;
 time_t last_lua_resv_update = (time_t) 0;
-
+static const char *req_fxns[] = {
+	"slurm_job_submit",
+	"slurm_job_modify",
+	NULL
+};
 /*
  *  Mutex for protecting multi-threaded access to this plugin.
  *   (Only 1 thread at a time should be in here)
@@ -174,6 +178,27 @@ static char *_get_default_qos(uint32_t user_id, char *account, char *partition)
 	}
 }
 
+/* Get the comment for an association (or NULL if not present) */
+static char *_get_assoc_comment(uint32_t user_id, char *account)
+{
+	slurmdb_assoc_rec_t assoc;
+	char *comment = NULL;
+
+	memset(&assoc, 0, sizeof(slurmdb_assoc_rec_t));
+	assoc.uid = user_id;
+	if (account) {
+		assoc.acct = account;
+	} else {
+		assoc.acct = _get_default_account(user_id);
+	}
+
+	if (assoc_mgr_fill_in_assoc(acct_db_conn, &assoc, accounting_enforce,
+				    NULL, false) != SLURM_ERROR)
+		comment = xstrdup(assoc.comment);
+
+	return comment;
+}
+
 /* Get fields in an existing slurmctld job_record */
 static int _job_rec_field_index(lua_State *L)
 {
@@ -249,9 +274,13 @@ static int _resv_field(const slurmctld_resv_t *resv_ptr,
 	} else if (!xstrcmp(name, "flags")) {
 		lua_pushnumber(L, resv_ptr->flags);
 	} else if (!xstrcmp(name, "full_nodes")) {
-		lua_pushboolean(L, resv_ptr->full_nodes);
+		lua_pushboolean(L,
+				(resv_ptr->ctld_flags & RESV_CTLD_FULL_NODE) ?
+				true : false);
 	} else if (!xstrcmp(name, "flags_set_node")) {
-		lua_pushboolean(L, resv_ptr->flags_set_node);
+		lua_pushboolean(L,
+				(resv_ptr->ctld_flags &
+				 RESV_CTLD_NODE_FLAGS_SET) ? true : false);
 	} else if (!xstrcmp(name, "licenses")) {
 		lua_pushstring(L, resv_ptr->licenses);
 	} else if (!xstrcmp(name, "node_cnt")) {
@@ -341,7 +370,11 @@ static int _set_job_env_field(lua_State *L)
 	if (job_desc == NULL) {
 		error("%s: job_desc is NULL", __func__);
 	} else if (job_desc->environment == NULL) {
-		error("%s: job_desc->environment is NULL", __func__);
+		if (job_desc->script)
+			error("%s: %s: job_desc->environment is NULL.",
+			      plugin_type, __func__);
+		else
+			info("job_desc->environment only accessible for batch jobs. ");
 		lua_pushnil(L);
 	} else {
 		value_str = luaL_checkstring(L, 3);
@@ -382,7 +415,11 @@ static int _job_env_field(const job_desc_msg_t *job_desc, const char *name)
 		error("%s: job_desc is NULL", __func__);
 		lua_pushnil(L);
 	} else if (job_desc->environment == NULL) {
-		error("%s: job_desc->environment is NULL", __func__);
+		if (job_desc->script)
+			error("%s: %s: job_desc->environment is NULL.",
+			      plugin_type, __func__);
+		else
+			info("job_desc->environment only accessible for batch jobs.");
 		lua_pushnil(L);
 	} else {
 		for (i = 0; job_desc->environment[i]; i++) {
@@ -472,6 +509,9 @@ static int _get_job_req_field(const job_desc_msg_t *job_desc, const char *name)
 		}
 	} else if (!xstrcmp(name, "array_inx")) {
 		lua_pushstring(L, job_desc->array_inx);
+	} else if (!xstrcmp(name, "assoc_comment")) {
+		lua_pushstring(L, _get_assoc_comment(job_desc->user_id,
+						     job_desc->account));
 	} else if (!xstrcmp(name, "batch_features")) {
 		lua_pushstring(L, job_desc->batch_features);
 	} else if (!xstrcmp(name, "begin_time")) {
@@ -486,6 +526,8 @@ static int _get_job_req_field(const job_desc_msg_t *job_desc, const char *name)
 		lua_pushstring(L, job_desc->clusters);
 	} else if (!xstrcmp(name, "comment")) {
 		lua_pushstring(L, job_desc->comment);
+	} else if (!xstrcmp(name, "container")) {
+		lua_pushstring(L, job_desc->container);
 	} else if (!xstrcmp(name, "contiguous")) {
 		lua_pushnumber(L, job_desc->contiguous);
 	} else if (!xstrcmp(name, "cores_per_socket")) {
@@ -500,6 +542,8 @@ static int _get_job_req_field(const job_desc_msg_t *job_desc, const char *name)
 		lua_pushnumber(L, job_desc->cpus_per_task);
 	} else if (!xstrcmp(name, "cpus_per_tres")) {
 		lua_pushstring(L, job_desc->cpus_per_tres);
+	} else if (!xstrcmp(name, "cron_job")) {
+		lua_pushboolean(L, job_desc->bitflags & CRON_JOB);
 	} else if (!xstrcmp(name, "default_account")) {
 		lua_pushstring(L, _get_default_account(job_desc->user_id));
 	} else if (!xstrcmp(name, "default_qos")) {
@@ -514,10 +558,10 @@ static int _get_job_req_field(const job_desc_msg_t *job_desc, const char *name)
 		lua_pushnumber(L, job_desc->end_time);
 	} else if (!xstrcmp(name, "environment")) {
 		_push_job_env((job_desc_msg_t *) job_desc); // No const
-	} else if (!xstrcmp(name, "extra")) {
-		lua_pushstring(L, job_desc->extra);
 	} else if (!xstrcmp(name, "exc_nodes")) {
 		lua_pushstring(L, job_desc->exc_nodes);
+	} else if (!xstrcmp(name, "extra")) {
+		lua_pushstring(L, job_desc->extra);
 	} else if (!xstrcmp(name, "features")) {
 		lua_pushstring(L, job_desc->features);
 	} else if (!xstrcmp(name, "gres")) {
@@ -553,16 +597,22 @@ static int _get_job_req_field(const job_desc_msg_t *job_desc, const char *name)
 		lua_pushnumber(L, job_desc->min_nodes);
 	} else if (!xstrcmp(name, "name")) {
 		lua_pushstring(L, job_desc->name);
+	} else if (!xstrcmp(name, "network")) {
+		lua_pushstring(L, job_desc->network);
 	} else if (!xstrcmp(name, "nice")) {
 		lua_pushnumber(L, job_desc->nice);
 	} else if (!xstrcmp(name, "ntasks_per_board")) {
 		lua_pushnumber(L, job_desc->ntasks_per_board);
 	} else if (!xstrcmp(name, "ntasks_per_core")) {
 		lua_pushnumber(L, job_desc->ntasks_per_core);
+	} else if (!xstrcmp(name, "ntasks_per_gpu")) {
+		lua_pushnumber(L, job_desc->ntasks_per_tres);
 	} else if (!xstrcmp(name, "ntasks_per_node")) {
 		lua_pushnumber(L, job_desc->ntasks_per_node);
 	} else if (!xstrcmp(name, "ntasks_per_socket")) {
 		lua_pushnumber(L, job_desc->ntasks_per_socket);
+	} else if (!xstrcmp(name, "ntasks_per_tres")) {
+		lua_pushnumber(L, job_desc->ntasks_per_tres);
 	} else if (!xstrcmp(name, "num_tasks")) {
 		lua_pushnumber(L, job_desc->num_tasks);
 		/* Continue support for old hetjob terminology. */
@@ -571,6 +621,8 @@ static int _get_job_req_field(const job_desc_msg_t *job_desc, const char *name)
 		lua_pushnumber(L, job_desc->het_job_offset);
 	} else if (!xstrcmp(name, "partition")) {
 		lua_pushstring(L, job_desc->partition);
+	} else if (!xstrcmp(name, "prefer")) {
+		lua_pushstring(L, job_desc->prefer);
 	} else if (!xstrcmp(name, "power_flags")) {
 		lua_pushnumber(L, job_desc->power_flags);
 	} else if (!xstrcmp(name, "pn_min_cpus")) {
@@ -589,6 +641,8 @@ static int _get_job_req_field(const job_desc_msg_t *job_desc, const char *name)
 		lua_pushstring(L, job_desc->qos);
 	} else if (!xstrcmp(name, "reboot")) {
 		lua_pushnumber(L, job_desc->reboot);
+	} else if (!xstrcmp(name, "req_context")) {
+		lua_pushstring(L, job_desc->req_context);
 	} else if (!xstrcmp(name, "req_nodes")) {
 		lua_pushstring(L, job_desc->req_nodes);
 	} else if (!xstrcmp(name, "req_switch")) {
@@ -599,7 +653,8 @@ static int _get_job_req_field(const job_desc_msg_t *job_desc, const char *name)
 		lua_pushstring(L, job_desc->reservation);
 	} else if (!xstrcmp(name, "script")) {
 		lua_pushstring(L, job_desc->script);
-	} else if (!xstrcmp(name, "shared")) {
+	} else if (!xstrcmp(name, "shared") ||
+		   !xstrcmp(name, "oversubscribe")) {
 		lua_pushnumber(L, job_desc->shared);
 	} else if (!xstrcmp(name, "site_factor")) {
 		if (job_desc->site_factor == NO_VAL)
@@ -751,6 +806,11 @@ static int _set_job_req_field(lua_State *L)
 		xfree(job_desc->comment);
 		if (strlen(value_str))
 			job_desc->comment = xstrdup(value_str);
+	} else if (!xstrcmp(name, "container")) {
+		value_str = luaL_checkstring(L, 3);
+		xfree(job_desc->container);
+		if (strlen(value_str))
+			job_desc->container = xstrdup(value_str);
 	} else if (!xstrcmp(name, "contiguous")) {
 		job_desc->contiguous = luaL_checknumber(L, 3);
 	} else if (!xstrcmp(name, "cores_per_socket")) {
@@ -797,7 +857,8 @@ static int _set_job_req_field(lua_State *L)
 		value_str = luaL_checkstring(L, 3);
 		xfree(job_desc->tres_per_node);
 		if (strlen(value_str))
-			job_desc->tres_per_node = xstrdup(value_str);
+			job_desc->tres_per_node =
+				gres_prepend_tres_type(value_str);
 	} else if (!xstrcmp(name, "immediate")) {
 		job_desc->immediate = luaL_checknumber(L, 3);
 	} else if (!xstrcmp(name, "licenses")) {
@@ -805,6 +866,13 @@ static int _set_job_req_field(lua_State *L)
 		xfree(job_desc->licenses);
 		if (strlen(value_str))
 			job_desc->licenses = xstrdup(value_str);
+	} else if (!xstrcmp(name, "mail_type")) {
+		job_desc->mail_type = luaL_checknumber(L, 3);
+	} else if (!xstrcmp(name, "mail_user")) {
+		value_str = luaL_checkstring(L, 3);
+		xfree(job_desc->mail_user);
+		if (strlen(value_str))
+			job_desc->mail_user = xstrdup(value_str);
 	} else if (!xstrcmp(name, "max_cpus")) {
 		job_desc->max_cpus = luaL_checknumber(L, 3);
 	} else if (!xstrcmp(name, "max_nodes")) {
@@ -830,10 +898,14 @@ static int _set_job_req_field(lua_State *L)
 			job_desc->name = xstrdup(value_str);
 	} else if (!xstrcmp(name, "nice")) {
 		job_desc->nice = luaL_checknumber(L, 3);
+	} else if (!xstrcmp(name, "ntasks_per_gpu")) {
+		job_desc->ntasks_per_tres = luaL_checknumber(L, 3);
 	} else if (!xstrcmp(name, "ntasks_per_node")) {
 		job_desc->ntasks_per_node = luaL_checknumber(L, 3);
 	} else if (!xstrcmp(name, "ntasks_per_socket")) {
 		job_desc->ntasks_per_socket = luaL_checknumber(L, 3);
+	} else if (!xstrcmp(name, "ntasks_per_tres")) {
+		job_desc->ntasks_per_tres = luaL_checknumber(L, 3);
 	} else if (!xstrcmp(name, "num_tasks")) {
 		job_desc->num_tasks = luaL_checknumber(L, 3);
 	} else if (!xstrcmp(name, "partition")) {
@@ -841,6 +913,11 @@ static int _set_job_req_field(lua_State *L)
 		xfree(job_desc->partition);
 		if (strlen(value_str))
 			job_desc->partition = xstrdup(value_str);
+	} else if (!xstrcmp(name, "prefer")) {
+		value_str = luaL_checkstring(L, 3);
+		xfree(job_desc->prefer);
+		if (strlen(value_str))
+			job_desc->prefer = xstrdup(value_str);
 	} else if (!xstrcmp(name, "power_flags")) {
 		job_desc->power_flags = luaL_checknumber(L, 3);
 	} else if (!xstrcmp(name, "pn_min_cpus")) {
@@ -881,7 +958,13 @@ static int _set_job_req_field(lua_State *L)
 		xfree(job_desc->script);
 		if (strlen(value_str))
 			job_desc->script = xstrdup(value_str);
-	} else if (!xstrcmp(name, "shared")) {
+	} else if (!xstrcmp(name, "selinux_context")) {
+		value_str = luaL_checkstring(L, 3);
+		xfree(job_desc->selinux_context);
+		if (strlen(value_str))
+			job_desc->selinux_context = xstrdup(value_str);
+	} else if (!xstrcmp(name, "shared") ||
+		   !xstrcmp(name, "oversubscribe")) {
 		job_desc->shared = luaL_checknumber(L, 3);
 	} else if (!xstrcmp(name, "site_factor")) {
 		job_desc->site_factor = luaL_checknumber(L, 3);
@@ -1037,6 +1120,8 @@ static int _part_rec_field(const part_record_t *part_ptr, const char *name)
 		lua_pushnumber(L, part_ptr->flags);
 	} else if (!xstrcmp(name, "max_cpus_per_node")) {
 		lua_pushnumber(L, part_ptr->max_cpus_per_node);
+	} else if (!xstrcmp(name, "max_cpus_per_socket")) {
+		lua_pushnumber(L, part_ptr->max_cpus_per_socket);
 	} else if (!xstrcmp(name, "max_mem_per_cpu") &&
 		   (part_ptr->max_mem_per_cpu != NO_VAL64) &&
 		   (part_ptr->max_mem_per_cpu & MEM_PER_CPU)) {
@@ -1049,7 +1134,8 @@ static int _part_rec_field(const part_record_t *part_ptr, const char *name)
 		lua_pushnumber(L, part_ptr->max_nodes);
 	} else if (!xstrcmp(name, "max_nodes_orig")) {
 		lua_pushnumber(L, part_ptr->max_nodes_orig);
-	} else if (!xstrcmp(name, "max_share")) {
+	} else if (!xstrcmp(name, "max_share") ||
+		   !xstrcmp(name, "max_oversubscribe")) {
 		lua_pushnumber(L, part_ptr->max_share);
 	} else if (!xstrcmp(name, "max_time")) {
 		lua_pushnumber(L, part_ptr->max_time);
@@ -1069,6 +1155,10 @@ static int _part_rec_field(const part_record_t *part_ptr, const char *name)
 		lua_pushstring(L, part_ptr->qos_char);
 	} else if (!xstrcmp(name, "state_up")) {
 		lua_pushnumber(L, part_ptr->state_up);
+	} else if (!xstrcmp(name, "total_cpus")) {
+		lua_pushnumber(L, part_ptr->total_cpus);
+	} else if (!xstrcmp(name, "total_nodes")) {
+		lua_pushnumber(L, part_ptr->total_nodes);
 	} else {
 		lua_pushnil(L);
 	}
@@ -1223,32 +1313,6 @@ static void _loadscript_extra(lua_State *st)
 	_register_lua_slurm_struct_functions(st);
 }
 
-static int _load_script(void)
-{
-	lua_State *load = NULL;
-	time_t load_time = lua_script_last_loaded;
-	const char *req_fxns[] = {
-		"slurm_job_submit",
-		"slurm_job_modify",
-		NULL
-	};
-
-	load = slurm_lua_loadscript(L, "job_submit/lua",
-				    lua_script_path, req_fxns, &load_time,
-				    _loadscript_extra);
-	if (load == L)
-		return SLURM_SUCCESS;
-	if (!load)
-		return SLURM_ERROR;
-
-	/* since complete finished error free, swap the states */
-	if (L)
-		lua_close(L);
-	L = load;
-	lua_script_last_loaded = load_time;
-	return SLURM_SUCCESS;
-}
-
 /*
  *  NOTE: The init callback should never be called multiple times,
  *   let alone called from multiple threads. Therefore, locking
@@ -1260,8 +1324,12 @@ int init(void)
 
 	if ((rc = slurm_lua_init()) != SLURM_SUCCESS)
 		return rc;
+	lua_script_path = get_extra_conf_path("job_submit.lua");
 
-	return _load_script();
+	return slurm_lua_loadscript(&L, "job_submit/lua",
+				    lua_script_path, req_fxns,
+				    &lua_script_last_loaded,
+				    _loadscript_extra);
 }
 
 int fini(void)
@@ -1272,6 +1340,7 @@ int fini(void)
 		L = NULL;
 		lua_script_last_loaded = 0;
 	}
+	xfree(lua_script_path);
 
 	slurm_lua_fini();
 
@@ -1283,10 +1352,14 @@ int fini(void)
 extern int job_submit(job_desc_msg_t *job_desc, uint32_t submit_uid,
 		      char **err_msg)
 {
-	int rc = SLURM_ERROR;
+	int rc;
 	slurm_mutex_lock (&lua_lock);
 
-	if ((rc = _load_script()))
+	rc = slurm_lua_loadscript(&L, "job_submit/lua",
+				  lua_script_path, req_fxns,
+				  &lua_script_last_loaded, _loadscript_extra);
+
+	if (rc != SLURM_SUCCESS)
 		goto out;
 
 	/*
@@ -1331,12 +1404,16 @@ out:	slurm_mutex_unlock (&lua_lock);
 
 /* Lua script hook called for "modify job" event. */
 extern int job_modify(job_desc_msg_t *job_desc, job_record_t *job_ptr,
-		      uint32_t submit_uid)
+		      uint32_t submit_uid, char **err_msg)
 {
-	int rc = SLURM_ERROR;
+	int rc;
 	slurm_mutex_lock (&lua_lock);
 
-	if ((rc = _load_script()))
+	rc = slurm_lua_loadscript(&L, "job_submit/lua",
+				  lua_script_path, req_fxns,
+				  &lua_script_last_loaded, _loadscript_extra);
+
+	if (rc == SLURM_ERROR)
 		goto out;
 
 	/*
@@ -1372,9 +1449,8 @@ extern int job_modify(job_desc_msg_t *job_desc, job_record_t *job_ptr,
 	slurm_lua_stack_dump(
 		"job_submit/lua", "job_modify, after lua_pcall", L);
 	if (user_msg) {
-		error("Use of log.user() in job_modify is not supported. "
-		      "Message discarded: (\"%s\")", user_msg);
-		xfree(user_msg);
+		*err_msg = user_msg;
+		user_msg = NULL;
 	}
 
 out:	slurm_mutex_unlock (&lua_lock);
