@@ -153,6 +153,16 @@ static void _set_tmpdirs(List lresp)
 	list_append(lresp, kvp);
 }
 
+static void _set_euid(list_t *lresp)
+{
+	pmix_info_t *kvp;
+
+	uid_t uid = pmixp_info_jobuid();
+
+	PMIXP_KVP_CREATE(kvp, PMIX_USERID, &uid, PMIX_UINT32);
+	list_append(lresp, kvp);
+}
+
 /*
  * information about relative ranks as assigned by the RM
  */
@@ -238,13 +248,13 @@ static void _set_procdatas(List lresp)
 		j = 0;
 		while ((tkvp = list_next(it))) {
 			/* Just copy all the fields here. We will free
-			 * original kvp's using list_destroy without free'ing
+			 * original kvp's using FREE_NULL_LIST without free'ing
 			 * their fields so it is safe to do so.
 			 */
 			info[j] = *tkvp;
 			j++;
 		}
-		list_destroy(rankinfo);
+		FREE_NULL_LIST(rankinfo);
 		PMIXP_KVP_ALLOC(kvp, PMIX_PROC_DATA);
 		PMIXP_INFO_ARRAY_CREATE(kvp, info, count);
 		info = NULL;
@@ -475,14 +485,16 @@ static void _set_localinfo(List lresp)
 extern int pmixp_libpmix_init(void)
 {
 	int rc;
+	mode_t rights = (S_IRUSR | S_IWUSR | S_IXUSR) |
+			(S_IRGRP | S_IWGRP | S_IXGRP);
 
-	if (0 != (rc = pmixp_mkdir(pmixp_info_tmpdir_lib()))) {
+	if (0 != (rc = pmixp_mkdir(pmixp_info_tmpdir_lib(), rights))) {
 		PMIXP_ERROR_STD("Cannot create server lib tmpdir: \"%s\"",
 				pmixp_info_tmpdir_lib());
 		return errno;
 	}
 
-	if (0 != (rc = pmixp_mkdir(pmixp_info_tmpdir_cli()))) {
+	if (0 != (rc = pmixp_mkdir(pmixp_info_tmpdir_cli(), rights))) {
 		PMIXP_ERROR_STD("Cannot create client cli tmpdir: \"%s\"",
 				pmixp_info_tmpdir_cli());
 		return errno;
@@ -495,7 +507,13 @@ extern int pmixp_libpmix_init(void)
 	}
 
 	/* TODO: must be deleted in future once info-key approach harden */
-	setenv(PMIXP_PMIXLIB_TMPDIR, pmixp_info_tmpdir_lib(), 1);
+	setenv(PMIXP_PMIXLIB_TMPDIR, _pmixp_info_client_tmpdir_lib(), 1);
+
+	/*
+	if( pmixp_fixrights(pmixp_info_tmpdir_lib(),
+		(uid_t) pmixp_info_jobuid(), rights) ){
+	}
+	*/
 
 	return 0;
 }
@@ -506,14 +524,14 @@ extern int pmixp_libpmix_finalize(void)
 
 	rc = pmixp_lib_finalize();
 
-	rc1 = rmdir_recursive(pmixp_info_tmpdir_lib(), true);
+	rc1 = pmixp_rmdir_recursively(pmixp_info_tmpdir_lib());
 	if (0 != rc1) {
 		PMIXP_ERROR_STD("Failed to remove %s\n",
 				pmixp_info_tmpdir_lib());
 		/* Not considering this as fatal error */
 	}
 
-	rc1 = rmdir_recursive(pmixp_info_tmpdir_cli(), true);
+	rc1 = pmixp_rmdir_recursively(pmixp_info_tmpdir_cli());
 	if (0 != rc1) {
 		PMIXP_ERROR_STD("Failed to remove %s\n",
 				pmixp_info_tmpdir_cli());
@@ -633,8 +651,10 @@ extern int pmixp_libpmix_job_set(void)
 
 	_set_topology(lresp);
 
+	_set_euid(lresp);
+
 	if (SLURM_SUCCESS != _set_mapsinfo(lresp)) {
-		list_destroy(lresp);
+		FREE_NULL_LIST(lresp);
 		PMIXP_ERROR("Can't build nodemap");
 		return SLURM_ERROR;
 	}
@@ -649,7 +669,7 @@ extern int pmixp_libpmix_job_set(void)
 		info[i] = *kvp;
 		i++;
 	}
-	list_destroy(lresp);
+	FREE_NULL_LIST(lresp);
 
 	register_caddy[0].active = 1;
 	rc = PMIx_server_register_nspace(pmixp_info_namespace(),
