@@ -64,7 +64,7 @@
 #include "src/common/assoc_mgr.h"
 #include "src/common/bitstring.h"
 #include "src/common/fd.h"
-#include "src/common/gres.h"
+#include "src/interfaces/gres.h"
 #include "src/common/list.h"
 #include "src/common/macros.h"
 #include "src/common/pack.h"
@@ -167,11 +167,9 @@ static int allowed_uid_cnt = 0;
 static uint32_t boot_time = (5 * 60);	/* 5 minute estimated boot time */
 static pthread_mutex_t config_mutex = PTHREAD_MUTEX_INITIALIZER;
 static uint32_t cpu_bind[KNL_NUMA_CNT];	/* Derived from numa_cpu_bind */
-static bool debug_flag = false;
 static uint16_t default_mcdram = KNL_CACHE;
 static uint16_t default_numa = KNL_ALL2ALL;
 static char *mc_path = NULL;
-static uint32_t node_reboot_weight = (INFINITE - 1);
 static char *numa_cpu_bind = NULL;
 static uint32_t syscfg_timeout = 0;
 static bool reconfig = false;
@@ -200,7 +198,6 @@ static s_p_options_t knl_conf_file_options[] = {
 	{"Force", S_P_UINT32},
 	{"LogFile", S_P_STRING},
 	{"McPath", S_P_STRING},
-	{"NodeRebootWeight", S_P_UINT32},
 	{"NumaCpuBind", S_P_STRING},
 	{"SyscfgPath", S_P_STRING},
 	{"SyscfgTimeout", S_P_UINT32},
@@ -235,7 +232,7 @@ static s_p_hashtbl_t *_config_make_tbl(char *filename)
 		return tbl;
 	}
 
-	if (s_p_parse_file(tbl, NULL, filename, false) == SLURM_ERROR) {
+	if (s_p_parse_file(tbl, NULL, filename, false, NULL) == SLURM_ERROR) {
 		error("knl.conf: %s: s_p_parse_file error: %m", __func__);
 		s_p_hashtbl_destroy(tbl);
 		tbl = NULL;
@@ -558,7 +555,7 @@ static void _update_cpu_bind(void)
 		      plugin_type, numa_cpu_bind);
 	}
 
-	if (debug_flag) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_NODE_FEATURES) {
 		for (i = 0; i < KNL_NUMA_CNT; i++) {
 			char cpu_bind_str[128], *numa_str;
 			if (cpu_bind[i] == 0)
@@ -577,7 +574,7 @@ static void _log_script_argv(char **script_argv, char *resp_msg)
 	char *cmd_line = NULL;
 	int i;
 
-	if (!debug_flag)
+	if (!(slurm_conf.debug_flags & DEBUG_FLAG_NODE_FEATURES))
 		return;
 
 	for (i = 0; script_argv[i]; i++) {
@@ -623,7 +620,7 @@ static char *_run_script(char *cmd_path, char **script_argv, int *status)
 		setpgid(0, 0);
 		execv(cmd_path, script_argv);
 		error("%s: execv(%s): %m", __func__, cmd_path);
-		exit(127);
+		_exit(127);
 	} else if (cpid < 0) {
 		close(pfd[0]);
 		close(pfd[1]);
@@ -708,7 +705,8 @@ static void _make_uid_array(char *uid_str)
 	tok = strtok_r(tmp_str, ",", &save_ptr);
 	while (tok) {
 		if (uid_from_string(tok, &allowed_uid[allowed_uid_cnt++]) < 0)
-			error("knl_generic.conf: Invalid AllowUserBoot: %s", tok);
+			fatal("knl_generic.conf: Invalid AllowUserBoot: %s",
+			      tok);
 		tok = strtok_r(NULL, ",", &save_ptr);
 	}
 	xfree(tmp_str);
@@ -807,7 +805,7 @@ extern int init(void)
 {
 	char *allow_mcdram_str, *allow_numa_str, *allow_user_str;
 	char *default_mcdram_str, *default_numa_str;
-	char *knl_conf_file, *tmp_str = NULL, *resume_program;
+	char *knl_conf_file, *tmp_str = NULL;
 	s_p_hashtbl_t *tbl;
 	struct stat stat_buf;
 	int i, rc = SLURM_SUCCESS;
@@ -825,7 +823,6 @@ extern int init(void)
 	for (i = 0; i < KNL_NUMA_CNT; i++)
 		cpu_bind[i] = 0;
 	syscfg_timeout = DEFAULT_SYSCFG_TIMEOUT;
-	debug_flag = false;
 	default_mcdram = KNL_CACHE;
 	default_numa = KNL_ALL2ALL;
 //FIXME: Need better mechanism to get MCDRAM percentages
@@ -838,9 +835,6 @@ extern int init(void)
 	mcdram_pct[4] = 0;	// KNL_AUTO
 	xfree(numa_cpu_bind);
 
-	if (slurm_get_debug_flags() & DEBUG_FLAG_NODE_FEATURES)
-		debug_flag = true;
-
 	knl_conf_file = get_extra_conf_path("knl_generic.conf");
 	if ((stat(knl_conf_file, &stat_buf) == 0) &&
 	    (tbl = _config_make_tbl(knl_conf_file))) {
@@ -883,8 +877,6 @@ extern int init(void)
 		}
 		(void) s_p_get_uint32(&force_load, "Force", tbl);
 		(void) s_p_get_string(&mc_path, "McPath", tbl);
-		(void) s_p_get_uint32(&node_reboot_weight, "NodeRebootWeight",
-				      tbl);
 		if (s_p_get_string(&numa_cpu_bind, "NumaCpuBind", tbl))
 			_update_cpu_bind();
 		(void) s_p_get_string(&syscfg_path, "SyscfgPath", tbl);
@@ -928,14 +920,13 @@ extern int init(void)
 		fclose(cpu_info_file);
 	}
 
-	if ((resume_program = slurm_get_resume_program())) {
+	if (slurm_conf.resume_program) {
 		error("Use of ResumeProgram with %s not currently supported",
 		      plugin_name);
-		xfree(resume_program);
 		rc = SLURM_ERROR;
 	}
 
-	if (slurm_get_debug_flags() & DEBUG_FLAG_NODE_FEATURES) {
+	if (slurm_conf.debug_flags & DEBUG_FLAG_NODE_FEATURES) {
 		allow_mcdram_str = _knl_mcdram_str(allow_mcdram);
 		allow_numa_str = _knl_numa_str(allow_numa);
 		allow_user_str = _make_uid_str(allowed_uid, allowed_uid_cnt);
@@ -949,7 +940,6 @@ extern int init(void)
 		     default_mcdram_str, default_numa_str);
 		info("Force=%u", force_load);
 		info("McPath=%s", mc_path);
-		info("NodeRebootWeight=%u", node_reboot_weight);
 		info("NumaCpuBind=%s", numa_cpu_bind);
 		info("SyscfgPath=%s (Found=%d)", syscfg_path, syscfg_found);
 		info("SyscfgTimeout=%u msec", syscfg_timeout);
@@ -961,7 +951,7 @@ extern int init(void)
 		xfree(default_mcdram_str);
 		xfree(default_numa_str);
 	}
-	gres_plugin_add("hbm");
+	gres_add("hbm");
 
 	if ((rc == SLURM_SUCCESS) &&
 	    ume_check_interval && running_in_slurmd()) {
@@ -985,7 +975,6 @@ extern int fini(void)
 	slurm_mutex_unlock(&ume_mutex);
 	xfree(allowed_uid);
 	allowed_uid_cnt = 0;
-	debug_flag = false;
 	xfree(mcdram_per_node);
 	xfree(mc_path);
 	xfree(numa_cpu_bind);
@@ -1283,7 +1272,7 @@ extern void node_features_p_node_state(char **avail_modes, char **current_mode)
 }
 
 /* Test if a job's feature specification is valid */
-extern int node_features_p_job_valid(char *job_features)
+extern int node_features_p_job_valid(char *job_features, list_t *feature_list)
 {
 	uint16_t job_mcdram, job_numa;
 	int mcdram_cnt, numa_cnt;
@@ -1331,15 +1320,18 @@ extern int node_features_p_job_valid(char *job_features)
  * Translate a job's feature request to the node features needed at boot time.
  *	If multiple MCDRAM or NUMA values are ORed, pick the first ones.
  * IN job_features - job's --constraint specification
- * RET features required on node reboot. Must xfree to release memory
+ * RET comma-delimited features required on node reboot. Must xfree to release
+ *     memory
  */
-extern char *node_features_p_job_xlate(char *job_features)
+extern char *node_features_p_job_xlate(char *job_features,
+				       list_t *feature_list,
+				       bitstr_t *job_node_bitmap)
 {
 	char *node_features = NULL;
 	char *tmp, *save_ptr = NULL, *mult, *sep = "", *tok;
 	bool has_numa = false, has_mcdram = false;
 
-	if ((job_features == NULL) || (job_features[0] ==  '\0'))
+	if (!job_features)
 		return node_features;
 
 	tmp = xstrdup(job_features);
@@ -1661,8 +1653,7 @@ extern bool node_features_p_node_power(void)
 extern int node_features_p_node_update(char *active_features,
 				       bitstr_t *node_bitmap)
 {
-	int i, i_first, i_last;
-	int rc = SLURM_SUCCESS, numa_inx = -1;
+	int i, rc = SLURM_SUCCESS, numa_inx = -1;
 	int mcdram_inx = 0;
 	uint64_t mcdram_size;
 	node_record_t *node_ptr;
@@ -1701,22 +1692,7 @@ extern int node_features_p_node_update(char *active_features,
 		mcdram_inx = -1;
 	}
 
-	xassert(node_bitmap);
-	i_first = bit_ffs(node_bitmap);
-	if (i_first >= 0)
-		i_last = bit_fls(node_bitmap);
-	else
-		i_last = i_first - 1;
-	for (i = i_first; i <= i_last; i++) {
-		if (!bit_test(node_bitmap, i))
-			continue;
-		if (i >= node_record_count) {
-			error("%s: Invalid node index (%d >= %d)",
-			      __func__, i, node_record_count);
-			rc = SLURM_ERROR;
-			break;
-		}
-		node_ptr = node_record_table_ptr + i;
+	for (i = 0; (node_ptr = next_node_bitmap(node_bitmap, &i)); i++) {
 		if ((numa_inx >= 0) && cpu_bind[numa_inx])
 			node_ptr->cpu_bind = cpu_bind[numa_inx];
 		if (mcdram_per_node && (mcdram_inx >= 0)) {
@@ -1725,9 +1701,9 @@ extern int node_features_p_node_update(char *active_features,
 			if (!node_ptr->gres)
 				node_ptr->gres =
 					xstrdup(node_ptr->config_ptr->gres);
-			gres_plugin_node_feature(node_ptr->name, "hbm",
-						 mcdram_size, &node_ptr->gres,
-						 &node_ptr->gres_list);
+			gres_node_feature(node_ptr->name, "hbm",
+					  mcdram_size, &node_ptr->gres,
+					  &node_ptr->gres_list);
 		}
 	}
 
@@ -2026,19 +2002,9 @@ extern void node_features_p_step_config(bool mem_sort, bitstr_t *numa_bitmap)
  * features */
 extern bool node_features_p_user_update(uid_t uid)
 {
-	static int reboot_allowed = -1;
 	int i;
 
-	if (reboot_allowed == -1) {
-		char *reboot_program = slurm_get_reboot_program();
-		if (reboot_program && reboot_program[0])
-			reboot_allowed = 1;
-		else
-			reboot_allowed = 0;
-		xfree(reboot_program);
-	}
-
-	if (reboot_allowed != 1) {
+	if (!slurm_conf.reboot_program || !slurm_conf.reboot_program[0]) {
 		info("Change in KNL mode not supported. No RebootProgram configured");
 		return false;
 	}
@@ -2050,6 +2016,8 @@ extern bool node_features_p_user_update(uid_t uid)
 		if (allowed_uid[i] == uid)
 			return true;
 	}
+	log_flag(NODE_FEATURES, "UID %u is not allowed to update node features",
+		 uid);
 
 	return false;
 }
@@ -2110,11 +2078,6 @@ extern void node_features_p_get_config(config_plugin_params_t *p)
 	key_pair->value = xstrdup(mc_path);
 	list_append(data, key_pair);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("NodeRebootWeight");
-	key_pair->value = xstrdup_printf("%u", node_reboot_weight);
-	list_append(data, key_pair);
-
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("SyscfgPath");
 	key_pair->value = xstrdup(syscfg_path);
@@ -2139,11 +2102,3 @@ extern void node_features_p_get_config(config_plugin_params_t *p)
 
 	return;
 }
-
-/*
- * Return node "weight" field if reboot required to change mode
- */
-extern uint32_t node_features_p_reboot_weight(void)
-{
-	return node_reboot_weight;
-}
