@@ -60,12 +60,12 @@
 #include "slurm/slurm_errno.h"
 
 #include "src/common/parse_time.h"
-#include "src/common/site_factor.h"
-#include "src/common/slurm_mcs.h"
-#include "src/common/slurm_priority.h"
+#include "src/interfaces/site_factor.h"
+#include "src/interfaces/mcs.h"
+#include "src/interfaces/priority.h"
 #include "src/common/slurm_time.h"
 #include "src/common/xstring.h"
-#include "src/common/gres.h"
+#include "src/interfaces/gres.h"
 
 #include "src/slurmctld/licenses.h"
 #include "src/slurmctld/read_config.h"
@@ -87,6 +87,7 @@ extern time_t last_job_update __attribute__((weak_import));
 extern slurm_conf_t slurm_conf __attribute__((weak_import));
 extern int slurmctld_tres_cnt __attribute__((weak_import));
 extern uint16_t accounting_enforce __attribute__((weak_import));
+extern int active_node_record_count __attribute__((weak_import));
 #else
 void *acct_db_conn = NULL;
 uint32_t cluster_cpus = NO_VAL;
@@ -95,6 +96,7 @@ time_t last_job_update = (time_t) 0;
 slurm_conf_t slurm_conf;
 int slurmctld_tres_cnt = 0;
 uint16_t accounting_enforce = 0;
+int active_node_record_count;
 #endif
 
 /*
@@ -152,6 +154,17 @@ static double decay_factor = 1; /* The decay factor when decaying time. */
 static void _priority_p_set_assoc_usage_debug(slurmdb_assoc_rec_t *assoc);
 static void _set_assoc_usage_efctv(slurmdb_assoc_rec_t *assoc);
 
+static void _destroy_priority_factors_obj_light(void *object)
+{
+	priority_factors_object_t *obj_ptr = object;
+
+	if (!obj_ptr)
+		return;
+
+	slurm_destroy_priority_factors(obj_ptr->prio_factors);
+	xfree(obj_ptr);
+}
+
 /*
  * apply decay factor to all associations usage_raw
  * IN: real_decay - decay to be applied to each associations' used
@@ -190,6 +203,14 @@ static int _apply_decay(double real_decay)
 		for (i=0; i<slurmctld_tres_cnt; i++)
 			assoc->usage->usage_tres_raw[i] *= real_decay;
 		assoc->usage->grp_used_wall *= real_decay;
+
+		if (assoc->leaf_usage && (assoc->leaf_usage != assoc->usage)) {
+			assoc->leaf_usage->usage_raw *= real_decay;
+			for (i = 0; i < slurmctld_tres_cnt; i++)
+				assoc->leaf_usage->usage_tres_raw[i] *=
+					real_decay;
+			assoc->leaf_usage->grp_used_wall *= real_decay;
+		}
 	}
 	list_iterator_destroy(itr);
 
@@ -238,6 +259,11 @@ static int _reset_usage(void)
 		for (i=0; i<slurmctld_tres_cnt; i++)
 			assoc->usage->usage_tres_raw[i] = 0;
 		assoc->usage->grp_used_wall = 0;
+
+		if (assoc->leaf_usage && (assoc->leaf_usage != assoc->usage)) {
+			slurmdb_destroy_assoc_usage(assoc->leaf_usage);
+			assoc->leaf_usage = NULL;
+		}
 	}
 	list_iterator_destroy(itr);
 
@@ -257,7 +283,7 @@ static int _reset_usage(void)
 static void _read_last_decay_ran(time_t *last_ran, time_t *last_reset)
 {
 	char *state_file;
-	Buf buffer;
+	buf_t *buffer;
 
 	xassert(last_ran);
 	xassert(last_reset);
@@ -281,7 +307,7 @@ static void _read_last_decay_ran(time_t *last_ran, time_t *last_reset)
 
 	safe_unpack_time(last_ran, buffer);
 	safe_unpack_time(last_reset, buffer);
-	free_buf(buffer);
+	FREE_NULL_BUFFER(buffer);
 	log_flag(PRIO, "Last ran decay on jobs at %ld", (long) *last_ran);
 
 	return;
@@ -290,7 +316,7 @@ unpack_error:
 	if (!ignore_state_errors)
 		fatal("Incomplete priority last decay file exiting, start with '-i' to ignore this. Warning: using -i will lose the data that can't be recovered.");
 	error("Incomplete priority last decay file returning");
-	free_buf(buffer);
+	FREE_NULL_BUFFER(buffer);
 	return;
 
 }
@@ -302,7 +328,7 @@ static int _write_last_decay_ran(time_t last_ran, time_t last_reset)
 	int error_code = SLURM_SUCCESS;
 	int state_fd;
 	char *old_file, *new_file, *state_file;
-	Buf buffer;
+	buf_t *buffer;
 
 	if (!xstrcmp(slurm_conf.state_save_location, "/dev/null")) {
 		error("Can not save priority state information, "
@@ -365,7 +391,7 @@ static int _write_last_decay_ran(time_t last_ran, time_t last_reset)
 
 	unlock_state_files();
 	debug4("done writing time %ld", (long)last_ran);
-	free_buf(buffer);
+	FREE_NULL_BUFFER(buffer);
 
 	return error_code;
 }
@@ -507,7 +533,7 @@ static uint32_t _get_priority_internal(time_t start_time,
 				       job_record_t *job_ptr)
 {
 	double priority	= 0.0;
-	priority_factors_object_t pre_factors;
+	priority_factors_t pre_factors;
 	uint64_t tmp_64;
 	double tmp_tres = 0.0;
 	char *multi_part_str = NULL;
@@ -517,7 +543,7 @@ static uint32_t _get_priority_internal(time_t start_time,
 			xfree(job_ptr->prio_factors->tres_weights);
 			xfree(job_ptr->prio_factors->priority_tres);
 			memset(job_ptr->prio_factors, 0,
-			       sizeof(priority_factors_object_t));
+			       sizeof(priority_factors_t));
 		}
 		return job_ptr->priority;
 	}
@@ -530,7 +556,7 @@ static uint32_t _get_priority_internal(time_t start_time,
 			xfree(job_ptr->prio_factors->tres_weights);
 			xfree(job_ptr->prio_factors->priority_tres);
 			memset(job_ptr->prio_factors, 0,
-			       sizeof(priority_factors_object_t));
+			       sizeof(priority_factors_t));
 		}
 		return 0;
 	}
@@ -539,7 +565,7 @@ static uint32_t _get_priority_internal(time_t start_time,
 
 	if (slurm_conf.debug_flags & DEBUG_FLAG_PRIO) {
 		memcpy(&pre_factors, job_ptr->prio_factors,
-		       sizeof(priority_factors_object_t));
+		       sizeof(priority_factors_t));
 		if (job_ptr->prio_factors->priority_tres) {
 			pre_factors.priority_tres = xcalloc(slurmctld_tres_cnt,
 							    sizeof(double));
@@ -548,7 +574,7 @@ static uint32_t _get_priority_internal(time_t start_time,
 			       sizeof(double) * slurmctld_tres_cnt);
 		}
 	} else	/* clang needs this memset to avoid a warning */
-		memset(&pre_factors, 0, sizeof(priority_factors_object_t));
+		memset(&pre_factors, 0, sizeof(priority_factors_t));
 
 	job_ptr->prio_factors->priority_age  *= (double)weight_age;
 	job_ptr->prio_factors->priority_assoc *= (double)weight_assoc;
@@ -1057,7 +1083,7 @@ static int _apply_new_usage(job_record_t *job_ptr, time_t start_period,
 				     job_ptr->tres_alloc_cnt[i]);
 			}
 		} else
-			info("No alloced TRES, state is %s",
+			info("No allocated TRES, state is %s",
 			     job_state_string(job_ptr->job_state));
 	}
 	/* get the time in decayed fashion */
@@ -1152,7 +1178,7 @@ static int _apply_new_usage(job_record_t *job_ptr, time_t start_period,
 	/* We want to do this all the way up
 	 * to and including root.  This way we
 	 * can keep track of how much usage
-	 * has occured on the entire system
+	 * has occurred on the entire system
 	 * and use that to normalize against. */
 	while (assoc) {
 		assoc->usage->grp_used_wall += run_decay;
@@ -1386,6 +1412,51 @@ static void *_decay_thread(void *no_data)
 	return NULL;
 }
 
+static priority_factors_object_t *_create_prio_factors_obj(
+	job_record_t *job_ptr, part_record_t *job_part_ptr)
+{
+	priority_factors_object_t *obj =
+		xmalloc(sizeof(priority_factors_object_t));
+
+	/*
+	 * Don't xstrdup dup anything here, the list is freed
+	 * with _destroy_priority_factors_obj_light.
+	 */
+	obj->account = job_ptr->account;
+	obj->job_id = job_ptr->job_id;
+	obj->partition = job_part_ptr ?
+		job_part_ptr->name : job_ptr->part_ptr->name;
+	obj->qos = job_ptr->qos_ptr ? job_ptr->qos_ptr->name : NULL;
+	obj->user_id = job_ptr->user_id;
+
+	if (job_ptr->direct_set_prio) {
+		obj->direct_prio = job_ptr->priority;
+	} else {
+		obj->prio_factors = xmalloc(sizeof(priority_factors_t));
+		slurm_copy_priority_factors(obj->prio_factors,
+					    job_ptr->prio_factors);
+
+		/* This portion is only needed for multi-partition jobs */
+		if (job_part_ptr) {
+			obj->prio_factors->priority_part =
+				((flags & PRIORITY_FLAGS_NO_NORMAL_PART) ?
+				 job_part_ptr->priority_job_factor :
+				 job_part_ptr->norm_priority) *
+				(double)weight_part;
+			if (obj->prio_factors->priority_tres) {
+				_get_tres_factors(
+					job_ptr, job_part_ptr,
+					obj->prio_factors->priority_tres);
+				_get_tres_prio_weighted(
+					obj->prio_factors->priority_tres);
+			}
+		}
+	}
+
+	return obj;
+}
+
+/* this function can be removed 2 versions after 23.02 */
 /* If the specified job record satisfies the filter specifications in req_msg
  * and part_ptr_list (partition name filters), then add its priority specs
  * to ret_list */
@@ -1393,10 +1464,9 @@ static void _filter_job(job_record_t *job_ptr,
 			priority_factors_request_msg_t *req_msg,
 			List part_ptr_list, List ret_list)
 {
-	priority_factors_object_t *obj = NULL;
 	part_record_t *job_part_ptr = NULL, *filter_part_ptr = NULL;
 	List req_job_list, req_user_list;
-	int filter = 0, inx;
+	int filter = 0;
 	ListIterator iterator, job_iter, filter_iter;
 	uint32_t *job_id;
 	uint32_t *user_id;
@@ -1459,23 +1529,13 @@ static void _filter_job(job_record_t *job_ptr,
 		}
 
 		if (filter == 0) {
-			obj = xmalloc(sizeof(priority_factors_object_t));
-			if (job_ptr->direct_set_prio) {
-				obj->direct_prio = job_ptr->priority;
-			} else {
-				slurm_copy_priority_factors_object(obj,
-							job_ptr->prio_factors);
-			}
-			obj->job_id = job_ptr->job_id;
-			obj->partition = job_part_ptr->name;
-			obj->user_id = job_ptr->user_id;
-			list_append(ret_list, obj);
+			list_append(ret_list,
+				    _create_prio_factors_obj(job_ptr, NULL));
 		}
 		return;
 	}
 
 	/* Filter by partition, job in multiple partitions */
-	inx = 0;
 	job_iter = list_iterator_create(job_ptr->part_ptr_list);
 	while ((job_part_ptr = list_next(job_iter))) {
 		filter = 0;
@@ -1492,27 +1552,10 @@ static void _filter_job(job_record_t *job_ptr,
 		}
 
 		if (filter == 0) {
-			obj = xmalloc(sizeof(priority_factors_object_t));
-			slurm_copy_priority_factors_object(obj,
-						job_ptr->prio_factors);
-			obj->priority_part =
-				((flags & PRIORITY_FLAGS_NO_NORMAL_PART) ?
-				 job_part_ptr->priority_job_factor :
-				 job_part_ptr->norm_priority) *
-				(double)weight_part;
-			obj->job_id = job_ptr->job_id;
-			obj->partition = job_part_ptr->name;
-			obj->user_id = job_ptr->user_id;
-
-			if (obj->priority_tres) {
-				_get_tres_factors(job_ptr, job_part_ptr,
-						  obj->priority_tres);
-				_get_tres_prio_weighted(obj->priority_tres);
-			}
-
-			list_append(ret_list, obj);
+			list_append(ret_list,
+				    _create_prio_factors_obj(job_ptr,
+							     job_part_ptr));
 		}
-		inx++;
 	}
 	list_iterator_destroy(job_iter);
 }
@@ -1696,8 +1739,6 @@ static void _set_usage_efctv(slurmdb_assoc_rec_t *assoc)
 int init ( void )
 {
 	/* Write lock on jobs, read lock on nodes and partitions */
-	slurmctld_lock_t job_write_lock =
-		{ NO_LOCK, WRITE_LOCK, READ_LOCK, READ_LOCK, NO_LOCK };
 
 	/* This means we aren't running from the controller so skip setup. */
 	if (cluster_cpus == NO_VAL) {
@@ -1709,7 +1750,6 @@ int init ( void )
 
 	/* Check to see if we are running a supported accounting plugin */
 	if (!slurm_with_slurmdbd()) {
-		time_t start_time = time(NULL);
 		if (weight_age)
 			error("PriorityWeightAge can only be used with SlurmDBD, ignoring");
 		if (weight_fs)
@@ -1717,19 +1757,7 @@ int init ( void )
 		calc_fairshare = 0;
 		weight_age = 0;
 		weight_fs = 0;
-
-		/* Initialize job priority factors for valid sprio output */
-		lock_slurmctld(job_write_lock);
-		list_for_each(
-			job_list,
-			(ListForF) _decay_apply_new_usage_and_weighted_factors,
-			&start_time);
-		unlock_slurmctld(job_write_lock);
 	} else if (assoc_mgr_root_assoc) {
-		if (!cluster_cpus)
-			fatal("We need to have a cluster cpu count "
-			      "before we can init the priority/multifactor "
-			      "plugin");
 		assoc_mgr_root_assoc->usage->usage_efctv = 1.0;
 
 		/* The decay_thread sets up some global variables that are
@@ -1758,7 +1786,7 @@ int init ( void )
 		calc_fairshare = 0;
 	}
 
-	site_factor_plugin_init();
+	site_factor_g_init();
 
 	debug("%s loaded", plugin_name);
 	return SLURM_SUCCESS;
@@ -1786,7 +1814,7 @@ int fini ( void )
 	if (decay_handler_thread)
 		pthread_join(decay_handler_thread, NULL);
 
-	site_factor_plugin_fini();
+	site_factor_g_fini();
 
 	return SLURM_SUCCESS;
 }
@@ -1896,23 +1924,22 @@ extern double priority_p_calc_fs_factor(long double usage_efctv,
 	return priority_fs;
 }
 
+/* req_msg can be removed 2 versions after 23.02 */
 extern List priority_p_get_priority_factors_list(
 	priority_factors_request_msg_t *req_msg, uid_t uid)
 {
 	List ret_list = NULL, part_filter_list = NULL;
-	ListIterator itr;
+	ListIterator itr, job_iter;
 	job_record_t *job_ptr = NULL;
-	part_record_t *part_ptr;
+	part_record_t *part_ptr, *job_part_ptr = NULL;
 	time_t start_time = time(NULL);
 	char *part_str, *tok, *last = NULL;
-	/* Read lock on jobs, nodes, and partitions */
-	slurmctld_lock_t job_read_lock =
-		{ NO_LOCK, READ_LOCK, READ_LOCK, READ_LOCK, NO_LOCK };
 
-	xassert(req_msg);
+	xassert(verify_lock(JOB_LOCK, READ_LOCK));
+	xassert(verify_lock(NODE_LOCK, READ_LOCK));
+	xassert(verify_lock(PART_LOCK, READ_LOCK));
 
-	lock_slurmctld(job_read_lock);
-	if (req_msg->partitions) {
+	if (req_msg && req_msg->partitions) {
 		part_filter_list = list_create(NULL);
 		part_str = xstrdup(req_msg->partitions);
 		tok = strtok_r(part_str, ",", &last);
@@ -1927,7 +1954,7 @@ extern List priority_p_get_priority_factors_list(
 	if (job_list && list_count(job_list)) {
 		time_t use_time;
 
-		ret_list = list_create(slurm_destroy_priority_factors_object);
+		ret_list = list_create(_destroy_priority_factors_obj_light);
 		itr = list_iterator_create(job_list);
 		while ((job_ptr = list_next(itr))) {
 			if (!(flags & PRIORITY_FLAGS_CALCULATE_RUNNING) &&
@@ -1962,18 +1989,48 @@ extern List priority_p_get_priority_factors_list(
 			      !assoc_mgr_is_user_acct_coord(acct_db_conn, uid,
 			                                    job_ptr->account))||
 			     ((slurm_mcs_get_privatedata() == 1) &&
-			      (mcs_g_check_mcs_label(uid, job_ptr->mcs_label)
-			       != 0))))
+			      (mcs_g_check_mcs_label(uid, job_ptr->mcs_label,
+						     false) != 0))))
 				continue;
 
-			_filter_job(job_ptr, req_msg, part_filter_list,
-				    ret_list);
+			/* this can be removed 2 versions after 23.02 */
+			if(req_msg) {
+				_filter_job(job_ptr, req_msg, part_filter_list,
+					    ret_list);
+				continue;
+			}
+
+			/*
+			 * Job is not in any partition, so there is nothing to
+			 * return. This can happen if the Partition was deleted,
+			 * CALCULATE_RUNNING is enabled, and this job is still
+			 * waiting out MinJobAge before being removed from the
+			 * system.
+			 */
+			if (!job_ptr->part_ptr && !job_ptr->part_ptr_list)
+				continue;
+
+			/* Job in one partition */
+			if (!job_ptr->part_ptr_list) {
+				list_append(ret_list,
+					    _create_prio_factors_obj(
+						    job_ptr, NULL));
+				continue;
+			}
+
+			/* Job in multiple partitions */
+			job_iter = list_iterator_create(job_ptr->part_ptr_list);
+			while ((job_part_ptr = list_next(job_iter))) {
+				list_append(ret_list,
+					    _create_prio_factors_obj(
+						    job_ptr, job_part_ptr));
+			}
+			list_iterator_destroy(job_iter);
 		}
 		list_iterator_destroy(itr);
 		if (!list_count(ret_list))
 			FREE_NULL_LIST(ret_list);
 	}
-	unlock_slurmctld(job_read_lock);
 	FREE_NULL_LIST(part_filter_list);
 
 	return ret_list;
@@ -2039,6 +2096,28 @@ extern int decay_apply_weighted_factors(job_record_t *job_ptr,
 	return SLURM_SUCCESS;
 }
 
+extern uint32_t priority_p_recover(uint32_t prio_boost)
+{
+	time_t start_time;
+	slurmctld_lock_t job_write_lock = {
+		.job = WRITE_LOCK,
+		.node = READ_LOCK,
+		.part = READ_LOCK,
+	};
+
+	if (slurm_with_slurmdbd())
+		return 0;
+
+	start_time = time(NULL);
+	/* Initialize job priority factors for valid sprio output */
+	lock_slurmctld(job_write_lock);
+	list_for_each(job_list,
+		      (ListForF) _decay_apply_new_usage_and_weighted_factors,
+		      &start_time);
+	unlock_slurmctld(job_write_lock);
+
+	return 0;
+}
 
 extern void set_priority_factors(time_t start_time, job_record_t *job_ptr)
 {
@@ -2048,12 +2127,11 @@ extern void set_priority_factors(time_t start_time, job_record_t *job_ptr)
 
 	if (!job_ptr->prio_factors) {
 		job_ptr->prio_factors =
-			xmalloc(sizeof(priority_factors_object_t));
+			xmalloc(sizeof(priority_factors_t));
 	} else {
 		xfree(job_ptr->prio_factors->tres_weights);
 		xfree(job_ptr->prio_factors->priority_tres);
-		memset(job_ptr->prio_factors, 0,
-		       sizeof(priority_factors_object_t));
+		memset(job_ptr->prio_factors, 0, sizeof(priority_factors_t));
 	}
 
 	if (weight_age && job_ptr->details->accrue_time) {
@@ -2079,8 +2157,10 @@ extern void set_priority_factors(time_t start_time, job_record_t *job_ptr)
 	}
 
 	/* FIXME: this should work off the product of TRESBillingWeights */
-	if (weight_js) {
+	if (weight_js && active_node_record_count && cluster_cpus) {
 		uint32_t cpu_cnt = 0, min_nodes = 1;
+		int node_count = active_node_record_count;
+
 		/* On the initial run of this we don't have total_cpus
 		   so go off the requesting.  After the first shot
 		   total_cpus should be filled in.
@@ -2101,7 +2181,7 @@ extern void set_priority_factors(time_t start_time, job_record_t *job_ptr)
 			job_ptr->prio_factors->priority_js =
 				(double)min_nodes *
 				(double)cluster_cpus /
-				(double)node_record_count;
+				(double)node_count;
 			if (cpu_cnt > job_ptr->prio_factors->priority_js) {
 				job_ptr->prio_factors->priority_js =
 					(double)cpu_cnt;
@@ -2120,9 +2200,12 @@ extern void set_priority_factors(time_t start_time, job_record_t *job_ptr)
 					job_ptr->prio_factors->priority_js;
 			}
 		} else if (slurm_conf.priority_favor_small) {
-			job_ptr->prio_factors->priority_js =
-				(double)(node_record_count - min_nodes)
-				/ (double)node_record_count;
+			if (node_count > min_nodes)
+				job_ptr->prio_factors->priority_js =
+					(double) (node_count - min_nodes) /
+					(double) node_count;
+			else
+				job_ptr->prio_factors->priority_js = 0;
 			if (cpu_cnt) {
 				job_ptr->prio_factors->priority_js +=
 					(double)(cluster_cpus - cpu_cnt)
@@ -2131,7 +2214,7 @@ extern void set_priority_factors(time_t start_time, job_record_t *job_ptr)
 			}
 		} else {	/* favor large */
 			job_ptr->prio_factors->priority_js =
-				(double)min_nodes / (double)node_record_count;
+				(double) min_nodes / (double) node_count;
 			if (cpu_cnt) {
 				job_ptr->prio_factors->priority_js +=
 					(double)cpu_cnt / (double)cluster_cpus;
