@@ -47,10 +47,12 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+#include "src/common/data.h"
 #include "src/common/read_config.h"
 #include "src/common/xstring.h"
 #include "src/common/proc_args.h"
 #include "src/common/uid.h"
+#include "src/interfaces/serializer.h"
 
 #include "src/squeue/squeue.h"
 
@@ -60,17 +62,18 @@
 #define OPT_LONG_HIDE         0x102
 #define OPT_LONG_START        0x103
 #define OPT_LONG_NOCONVERT    0x104
-#define OPT_LONG_ARRAY_UNIQUE 0x105
 #define OPT_LONG_LOCAL        0x106
 #define OPT_LONG_SIBLING      0x107
 #define OPT_LONG_FEDR         0x108
 #define OPT_LONG_ME           0x109
+#define OPT_LONG_JSON         0x110
+#define OPT_LONG_YAML         0x111
+#define OPT_LONG_AUTOCOMP     0x112
 
 /* FUNCTIONS */
 static List  _build_job_list( char* str );
 static List  _build_str_list( char* str );
 static List  _build_state_list( char* str );
-static List  _build_all_states_list( void );
 static List  _build_step_list( char* str );
 static List  _build_user_list( char* str );
 static char *_get_prefix(char *token);
@@ -82,8 +85,10 @@ static void _parse_long_token( char *token, char *sep, int *field_size,
 			       bool *right_justify, char **suffix);
 static void  _print_options( void );
 static void  _usage( void );
-static bool _check_node_names(hostset_t);
-static bool _find_a_host(char *, node_info_msg_t *);
+static void _filter_nodes(void);
+static List _load_clusters_nodes(void);
+static void _node_info_list_del(void *data);
+static char *_map_node_name(List clusters_node_info, char *name);
 
 /*
  * parse_command_line
@@ -96,10 +101,10 @@ parse_command_line( int argc, char* *argv )
 	int opt_char;
 	int option_index;
 	static struct option long_options[] = {
+		{"autocomplete", required_argument, 0, OPT_LONG_AUTOCOMP},
 		{"accounts",   required_argument, 0, 'A'},
 		{"all",        no_argument,       0, 'a'},
 		{"array",      no_argument,       0, 'r'},
-		{"array-unique",no_argument,      0, OPT_LONG_ARRAY_UNIQUE},
 		{"Format",     required_argument, 0, 'O'},
 		{"format",     required_argument, 0, 'o'},
 		{"federation", no_argument,       0, OPT_LONG_FEDR},
@@ -134,6 +139,8 @@ parse_command_line( int argc, char* *argv )
 		{"users",      required_argument, 0, 'u'},
 		{"verbose",    no_argument,       0, 'v'},
 		{"version",    no_argument,       0, 'V'},
+		{"json", no_argument, 0, OPT_LONG_JSON},
+		{"yaml", no_argument, 0, OPT_LONG_YAML},
 		{NULL,         0,                 0, 0}
 	};
 
@@ -148,8 +155,6 @@ parse_command_line( int argc, char* *argv )
 		params.array_flag = true;
 	if ( ( env_val = getenv("SQUEUE_SORT") ) )
 		params.sort = xstrdup(env_val);
-	if (getenv("SQUEUE_ARRAY_UNIQUE"))
-		params.array_unique_flag = true;
 	if ( ( env_val = getenv("SLURM_CLUSTERS") ) ) {
 		if (!(params.clusters = slurmdb_get_info_cluster(env_val))) {
 			print_db_notok(env_val, 1);
@@ -316,9 +321,6 @@ parse_command_line( int argc, char* *argv )
 				exit(1);
 			}
 			break;
-		case OPT_LONG_ARRAY_UNIQUE:
-			params.array_unique_flag = true;
-			break;
 		case OPT_LONG_HELP:
 			_help();
 			exit(0);
@@ -349,6 +351,20 @@ parse_command_line( int argc, char* *argv )
 		case OPT_LONG_USAGE:
 			_usage();
 			exit(0);
+		case OPT_LONG_JSON:
+			params.mimetype = MIME_TYPE_JSON;
+			data_init();
+			serializer_g_init(MIME_TYPE_JSON_PLUGIN, NULL);
+			break;
+		case OPT_LONG_YAML:
+			params.mimetype = MIME_TYPE_YAML;
+			data_init();
+			serializer_g_init(MIME_TYPE_YAML_PLUGIN, NULL);
+			break;
+		case OPT_LONG_AUTOCOMP:
+			suggest_completion(long_options, optarg);
+			exit(0);
+			break;
 		}
 	}
 
@@ -388,41 +404,8 @@ parse_command_line( int argc, char* *argv )
 		}
 	}
 
-	if ( params.nodes ) {
-		char *name1 = NULL;
-		char *name2 = NULL;
-		hostset_t nodenames = hostset_create(NULL);
-
-		while ( hostset_count(params.nodes) > 0 ) {
-			name1 = hostset_pop(params.nodes);
-
-			/* localhost = use current host name */
-			if ( xstrcasecmp("localhost", name1) == 0 ) {
-				name2 = xmalloc(128);
-				gethostname_short(name2, 128);
-			} else {
-				/* translate NodeHostName to NodeName */
-				name2 = slurm_conf_get_nodename(name1);
-
-				/* use NodeName if translation failed */
-				if ( name2 == NULL )
-					name2 = xstrdup(name1);
-			}
-			hostset_insert(nodenames, name2);
-			free(name1);
-			xfree(name2);
-		}
-
-		/* Replace params.nodes with the new one */
-		hostset_destroy(params.nodes);
-		params.nodes = nodenames;
-		/* Check if all node names specified
-		 * with -w are known to the controller.
-		 */
-		if (!_check_node_names(params.nodes)) {
-			exit(1);
-		}
-	}
+	if ( params.nodes )
+		_filter_nodes();
 
 	if ( ( params.accounts == NULL ) &&
 	     ( env_val = getenv("SQUEUE_ACCOUNT") ) ) {
@@ -854,12 +837,6 @@ extern int parse_format( char* format )
 							    field_size,
 							    right_justify,
 							    suffix );
-			else if (field[0] == 's')
-				job_format_add_select_jobinfo(
-					params.format_list,
-					field_size,
-					right_justify,
-					suffix );
 			else if (field[0] == 'S')
 				job_format_add_time_start( params.format_list,
 							   field_size,
@@ -985,6 +962,18 @@ extern int parse_long_format( char* format_long )
 							     field_size,
 							     right_justify,
 							     suffix);
+			else if (!xstrncasecmp(token, "container",
+					       strlen("container")))
+				step_format_add_container(params.format_list,
+							  field_size,
+							  right_justify,
+							  suffix);
+			else if (!xstrncasecmp(token, "containerid",
+					       strlen("containerid")))
+				step_format_add_container_id(params.format_list,
+							     field_size,
+							     right_justify,
+							     suffix);
 			else if (!xstrncasecmp(token, "numtasks",
 					       strlen("numtask")))
 				step_format_add_num_tasks( params.format_list,
@@ -1144,6 +1133,11 @@ extern int parse_long_format( char* format_long )
 							field_size,
 							right_justify,
 							suffix  );
+			else if (!xstrcasecmp(token, "container"))
+				job_format_add_container(params.format_list,
+						       field_size,
+						       right_justify,
+						       suffix );
 			else if (!xstrcasecmp(token, "jobid"))
 				job_format_add_job_id2(params.format_list,
 						       field_size,
@@ -1176,6 +1170,15 @@ extern int parse_long_format( char* format_long )
 							    field_size,
 							    right_justify,
 							    suffix);
+			else if (!xstrcasecmp(token, "container"))
+				job_format_add_container(params.format_list,
+							 field_size,
+							 right_justify, suffix);
+			else if (!xstrcasecmp(token, "containerid"))
+				job_format_add_container_id(params.format_list,
+							    field_size,
+							    right_justify,
+							    suffix);
 			else if (!xstrcasecmp(token, "delayboot"))
 				job_format_add_delay_boot(params.format_list,
 							  field_size,
@@ -1229,6 +1232,11 @@ extern int parse_long_format( char* format_long )
 							params.format_list,
 							field_size,
 							right_justify, suffix);
+			else if (!xstrcasecmp(token, "prefer"))
+				job_format_add_prefer(params.format_list,
+						      field_size,
+						      right_justify,
+						      suffix);
 			else if (!xstrcasecmp(token, "arrayjobid"))
 				job_format_add_array_job_id(
 					params.format_list,
@@ -1367,12 +1375,6 @@ extern int parse_long_format( char* format_long )
 							    field_size,
 							    right_justify,
 							    suffix );
-			else if (!xstrcasecmp(token, "selectjobinfo"))
-				job_format_add_select_jobinfo(
-					params.format_list,
-					field_size,
-					right_justify,
-					suffix );
 			else if (!xstrcasecmp(token, "starttime"))
 				job_format_add_time_start( params.format_list,
 							   field_size,
@@ -1865,7 +1867,9 @@ _print_options(void)
 	printf( "users       = %s\n", params.users );
 	printf( "verbose     = %d\n", params.verbose );
 
-	if ((params.verbose > 1) && params.job_list) {
+	if (params.verbose <= 1)
+		goto endit;
+	if (params.job_list) {
 		i = 0;
 		iterator = list_iterator_create( params.job_list );
 		while ( (job_step_id = list_next( iterator )) ) {
@@ -1882,7 +1886,7 @@ _print_options(void)
 	}
 
 
-	if ((params.verbose > 1) && params.name_list) {
+	if (params.name_list) {
 		i = 0;
 		iterator = list_iterator_create( params.name_list );
 		while ( (name = list_next( iterator )) ) {
@@ -1891,7 +1895,7 @@ _print_options(void)
 		list_iterator_destroy( iterator );
 	}
 
-	if ((params.verbose > 1) && params.licenses_list) {
+	if (params.licenses_list) {
 		i = 0;
 		iterator = list_iterator_create( params.licenses_list );
 		while ( (license = list_next( iterator )) ) {
@@ -1900,7 +1904,7 @@ _print_options(void)
 		list_iterator_destroy( iterator );
 	}
 
-	if ((params.verbose > 1) && params.part_list) {
+	if (params.part_list) {
 		i = 0;
 		iterator = list_iterator_create( params.part_list );
 		while ( (part = list_next( iterator )) ) {
@@ -1909,7 +1913,9 @@ _print_options(void)
 		list_iterator_destroy( iterator );
 	}
 
-	if ((params.verbose > 1) && params.state_list) {
+	if (params.all_states) {
+		printf( "state_list = all\n");
+	} else if (params.state_list) {
 		i = 0;
 		iterator = list_iterator_create( params.state_list );
 		while ( (state_id = list_next( iterator )) ) {
@@ -1919,7 +1925,7 @@ _print_options(void)
 		list_iterator_destroy( iterator );
 	}
 
-	if ((params.verbose > 1) && params.step_list) {
+	if (params.step_list) {
 		char tmp_char[34];
 		i = 0;
 		iterator = list_iterator_create( params.step_list );
@@ -1946,7 +1952,7 @@ _print_options(void)
 		list_iterator_destroy( iterator );
 	}
 
-	if ((params.verbose > 1) && params.user_list) {
+	if (params.user_list) {
 		i = 0;
 		iterator = list_iterator_create( params.user_list );
 		while ( (user = list_next( iterator )) ) {
@@ -1954,7 +1960,7 @@ _print_options(void)
 		}
 		list_iterator_destroy( iterator );
 	}
-
+endit:
 	printf( "-----------------------------\n\n\n" );
 } ;
 
@@ -2039,8 +2045,11 @@ _build_state_list( char* str )
 
 	if (str == NULL)
 		return NULL;
-	if (xstrcasecmp( str, "all") == 0)
-		return _build_all_states_list ();
+	if (!xstrcasecmp(str, "all")) {
+		params.all_states = true;
+		return NULL;
+	}
+	params.all_states = false;
 
 	my_list = list_create(NULL);
 	my_state_list = xstrdup(str);
@@ -2057,38 +2066,6 @@ _build_state_list( char* str )
 
 }
 
-static void _append_state_list(List my_list, uint32_t state_id)
-{
-	uint32_t *state_rec;
-
-	state_rec = xmalloc(sizeof(uint32_t));
-	*state_rec = state_id;
-	list_append(my_list, state_rec);
-}
-
-/*
- * _build_all_states_list - build a list containing all possible job states
- * RET List of uint16_t values
- */
-static List
-_build_all_states_list( void )
-{
-	List my_list;
-	uint32_t i;
-
-	my_list = list_create( NULL );
-	for (i = 0; i < JOB_END; i++)
-		_append_state_list(my_list, i);
-
-	_append_state_list(my_list, JOB_COMPLETING);
-	_append_state_list(my_list, JOB_CONFIGURING);
-	_append_state_list(my_list, JOB_REVOKED);
-	_append_state_list(my_list, JOB_SPECIAL_EXIT);
-
-	return my_list;
-
-}
-
 /*
  * _build_step_list- build a list of job/step_ids
  * IN str - comma separated list of job_id[array_id].step_id values
@@ -2201,6 +2178,7 @@ Usage: squeue [OPTIONS]\n\
   -i, --iterate=seconds           specify an interation period\n\
   -j, --job=job(s)                comma separated list of jobs IDs\n\
                                   to view, default is all\n\
+      --json                      Produce JSON output\n\
       --local                     Report information only about jobs on the\n\
                                   local cluster. Overrides --federation.\n\
   -l, --long                      long report\n\
@@ -2234,63 +2212,138 @@ Usage: squeue [OPTIONS]\n\
   -V, --version                   output version information and exit\n\
   -w, --nodelist=hostlist         list of nodes to view, default is \n\
 				  all nodes\n\
+      --yaml                      Produce YAML output\n\
 \nHelp options:\n\
   --help                          show this help message\n\
   --usage                         display a brief summary of squeue options\n");
 }
 
-/* _check_node_names()
+/*
+ * Validate and assign filtered nodes to params.nodes.
  */
-static bool
-_check_node_names(hostset_t names)
+static void _filter_nodes(void)
 {
-	int cc;
-	node_info_msg_t *node_info;
-	char *host;
-	hostlist_iterator_t itr;
+	char *name = NULL, *nodename = NULL;
+	hostset_t nodenames = hostset_create(NULL);
+	List clusters_nodes = NULL;
 
-	if (names == NULL)
-		return true;
+	/* Retrieve node_info from controllers */
+	if (!(clusters_nodes = _load_clusters_nodes()))
+		exit(1);
 
-	cc = slurm_load_node(0,
-			     &node_info,
-			     SHOW_ALL);
-	if (cc != 0) {
-		slurm_perror ("slurm_load_node error");
-		return false;
-	}
-
-	itr = hostset_iterator_create(names);
-	while ((host = hostlist_next(itr))) {
-		if (!_find_a_host(host, node_info)) {
-			error("Invalid node name %s", host);
-			free(host);
-			hostlist_iterator_destroy(itr);
-			return false;
+	/* Map all node names specified with -w, if known to any controller. */
+	while ((name = hostset_shift(params.nodes))) {
+		if (!(nodename = _map_node_name(clusters_nodes, name))) {
+			free(name);
+			hostset_destroy(params.nodes);
+			FREE_NULL_LIST(clusters_nodes);
+			exit(1);
 		}
-		free(host);
+		hostset_insert(nodenames, nodename);
+		free(name);
+		xfree(nodename);
 	}
-	hostlist_iterator_destroy(itr);
+	FREE_NULL_LIST(clusters_nodes);
 
-	return true;
+	/* Replace params.nodes with the new one */
+	hostset_destroy(params.nodes);
+	params.nodes = nodenames;
 }
 
-/* _find_a_host()
+/*
+ * ListDelF for a list of node_info_msg_t.
  */
-static bool
-_find_a_host(char *host, node_info_msg_t *node)
+static void _node_info_list_del(void *data)
 {
-	int cc;
+	node_info_msg_t *node_info_ptr = data;
 
-	for (cc = 0; cc < node->record_count; cc++) {
-		/* This can happen if the host is removed
-		 * fron DNS but still in slurm.conf
-		 */
-		if (node->node_array[cc].name == NULL)
-			continue;
-		if (xstrcmp(host, node->node_array[cc].name) == 0)
-			return true;
+	slurm_free_node_info_msg(node_info_ptr);
+}
+
+/*
+ * Retrieve node_info_msg_t for params.clusters or just local cluster.
+ * RET: List of all needed node_info_msg_t or NULL if any fail
+ *
+ * NOTE: caller must free the returned list if not NULL.
+ */
+static List _load_clusters_nodes(void)
+{
+	List node_info_list = NULL;
+	ListIterator iter = NULL;
+	node_info_msg_t *node_info = NULL;
+
+	node_info_list = list_create(_node_info_list_del);
+
+	if (params.clusters)
+		iter = list_iterator_create(params.clusters);
+
+	do {
+		if (slurm_load_node(0, &node_info, SHOW_ALL)) {
+			slurm_perror("slurm_load_node error");
+			FREE_NULL_LIST(node_info_list);
+			break;
+		}
+
+		list_append(node_info_list, node_info);
+	} while (params.clusters && (working_cluster_rec = list_next(iter)));
+
+	/*
+	 * Don't need to reset working_cluster_rec here. Nobody uses it in
+	 * parse_command_line(), and it's already reset later in main().
+	 */
+	if (params.clusters)
+		list_iterator_destroy(iter);
+
+	return node_info_list;
+}
+
+/*
+ * Map name into NodeName, and handle the special "localhost" case.
+ * IN: pointer to an array of pointers to node_info_msg_t
+ * IN: input node name
+ * RET: mapped node name if valid, NULL otherwise
+ *
+ * NOTE: caller must xfree() the returned name.
+ */
+static char *_map_node_name(List clusters_node_info, char *name)
+{
+	char *nodename = NULL;
+	node_info_msg_t *node_info;
+	ListIterator node_info_itr;
+
+	if (!name)
+		return NULL;
+
+	/* localhost = use current host name */
+	if (!xstrcasecmp("localhost", name)) {
+		nodename = xmalloc(128);
+		gethostname_short(nodename, 128);
+	} else
+		nodename = xstrdup(name);
+
+	node_info_itr = list_iterator_create(clusters_node_info);
+
+	while ((node_info = list_next(node_info_itr))) {
+		for (int cc = 0; cc < node_info->record_count; cc++) {
+			/*
+			 * This can happen if the host is removed from DNS but
+			 * still in slurm.conf
+			 */
+			if (!node_info->node_array[cc].name)
+				continue;
+			if (!xstrcmp(nodename,
+				     node_info->node_array[cc].name) ||
+			    !xstrcmp(nodename,
+				     node_info->node_array[cc].node_hostname)) {
+				xfree(nodename);
+				list_iterator_destroy(node_info_itr);
+				return xstrdup(node_info->node_array[cc].name);
+			}
+		}
 	}
 
-	return false;
+	error("Invalid node name %s", name);
+	xfree(nodename);
+	list_iterator_destroy(node_info_itr);
+	return NULL;
 }
